@@ -19,6 +19,7 @@
 #include <iomanip>
 #include <ostream>
 
+#include <boost/foreach.hpp>
 #include <boost/format.hpp>
 #include <boost/regex.hpp>
 
@@ -26,6 +27,7 @@
 #include <core/FilePath.hpp>
 #include <core/FileSerializer.hpp>
 #include <core/SafeConvert.hpp>
+#include <core/RegexUtils.hpp>
 #include <core/StringUtils.hpp>
 #include <core/YamlUtil.hpp>
 #include <core/text/DcfParser.hpp>
@@ -269,7 +271,7 @@ bool interpretRVersionValue(const std::string& value,
    RVersionInfo version = rVersionFromString(value);
 
    if (version.number != kRVersionDefault &&
-       !boost::regex_match(version.number, boost::regex("[\\d\\.]+")))
+       !regex_utils::match(version.number, boost::regex("[\\d\\.]+")))
    {
       return false;
    }
@@ -310,6 +312,52 @@ std::ostream& operator << (std::ostream& stream, const YesNoAskValue& val)
    return stream ;
 }
 
+Error findProjectFile(FilePath filePath,
+                      FilePath* pProjPath)
+{
+   // check to see if we already have a .Rproj file
+   if (filePath.extensionLowerCase() == ".rproj")
+   {
+      *pProjPath = filePath;
+      return Success();
+   }
+
+   if (!filePath.isDirectory())
+      filePath = filePath.parent();
+
+   // no .Rproj file found; scan parent directories
+   for (; filePath.exists(); filePath = filePath.parent())
+   {
+      // scan this directory for .Rproj files
+      FilePath projPath = projectFromDirectory(filePath);
+      if (!projPath.empty())
+      {
+         *pProjPath = projPath;
+         return Success();
+      }
+   }
+
+   return fileNotFoundError(ERROR_LOCATION);
+}
+
+Error findProjectConfig(FilePath filePath,
+                        RProjectConfig* pConfig)
+{
+   Error error;
+   
+   FilePath projPath;
+   error = findProjectFile(filePath, &projPath);
+   if (error)
+      return error;
+   
+   std::string errorMessage;
+   error = readProjectFile(projPath, pConfig, &errorMessage);
+   if (error)
+      return error;
+   
+   return Success();
+}
+
 Error readProjectFile(const FilePath& projectFilePath,
                       RProjectConfig* pConfig,
                       std::string* pUserErrMsg)
@@ -697,6 +745,30 @@ Error readProjectFile(const FilePath& projectFilePath,
       pConfig->tutorialPath = "";
    }
 
+   // extract quit child processes on exit
+   it = dcfFields.find("QuitChildProcessesOnExit");
+   if (it != dcfFields.end())
+   {
+      if (!interpretYesNoAskValue(it->second, false, &(pConfig->quitChildProcessesOnExit)))
+         return requiredFieldError("QuitChildProcessesOnExit", pUserErrMsg);
+   }
+   else
+   {
+      pConfig->quitChildProcessesOnExit = defaultConfig.quitChildProcessesOnExit;
+      *pProvidedDefaults = true;
+   }
+
+   // extract default open docs
+   it = dcfFields.find("DefaultOpenDocs");
+   if (it != dcfFields.end())
+   {
+      pConfig->defaultOpenDocs = it->second;
+   }
+   else
+   {
+      pConfig->defaultOpenDocs = "";
+   }
+
    return Success();
 }
 
@@ -880,6 +952,20 @@ Error writeProjectFile(const FilePath& projectFilePath,
       contents.append(boost::str(tutorialFmt % config.tutorialPath));
    }
 
+   // add QuitChildProcessesOnExit if it's not the default
+   if (config.quitChildProcessesOnExit != DefaultValue)
+   {
+      boost::format quitChildProcFmt("\nQuitChildProcessesOnExit: %1%\n");
+      contents.append(boost::str(quitChildProcFmt % yesNoAskValueToString(config.quitChildProcessesOnExit)));
+   }
+
+   // add default open docs if it's present
+   if (!config.defaultOpenDocs.empty())
+   {
+      boost::format docsFmt("\nDefaultOpenDocs: %1%\n");
+      contents.append(boost::str(docsFmt % config.defaultOpenDocs));
+   }
+
    // write it
    return writeStringToFile(projectFilePath,
                             contents,
@@ -982,7 +1068,7 @@ bool isWebsiteDirectory(const FilePath& projectDir)
       {
          static const boost::regex reSite("^site:.*$");
          std::string yaml = yaml::extractYamlHeader(indexFile);
-         return boost::regex_search(yaml.begin(), yaml.end(), reSite);
+         return regex_utils::search(yaml.begin(), yaml.end(), reSite);
       }
    }
    else
