@@ -1,7 +1,7 @@
 /*
  * PosixChildProcess.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2009-17 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,7 @@
  */
 
 #include "ChildProcess.hpp"
+#include "ChildProcessSubprocPoll.hpp"
 
 #include <fcntl.h>
 #include <signal.h>
@@ -54,6 +55,19 @@ const int READ = 0;
 const int WRITE = 1;
 const std::size_t READ_ERR = -1;
 
+// how long we keep "saw activity" state at true even if we haven't seen
+// new activity
+const boost::posix_time::milliseconds kResetRecentDelay =
+                                         boost::posix_time::milliseconds(1000);
+
+// how often we update "has subprocesses" flag
+const boost::posix_time::milliseconds kCheckSubprocDelay =
+                                         boost::posix_time::milliseconds(200);
+
+// how often we query and store current working directory of subprocess
+const boost::posix_time::milliseconds kCheckCwdDelay =
+                                         boost::posix_time::milliseconds(2000);
+
 int resolveExitStatus(int status)
 {
    return WIFEXITED(status) ? WEXITSTATUS(status) : status;
@@ -147,7 +161,7 @@ struct ChildProcess::Impl
    {
    }
 
-   pid_t pid;
+   PidType pid;
    int fdStdin;
    int fdStdout;
    int fdStderr;
@@ -156,7 +170,7 @@ struct ChildProcess::Impl
    int fdMaster;
    char ctrlC;
 
-   void init(pid_t pid, int fdStdin, int fdStdout, int fdStderr)
+   void init(PidType pid, int fdStdin, int fdStdout, int fdStderr)
    {
       this->pid = pid;
       this->fdStdin = fdStdin;
@@ -165,7 +179,7 @@ struct ChildProcess::Impl
       this->fdMaster = -1;
    }
 
-   void init(pid_t pid, int fdMaster)
+   void init(PidType pid, int fdMaster)
    {
       this->pid = pid;
       this->fdStdin = fdMaster;
@@ -239,6 +253,20 @@ void ChildProcess::init(const std::string& command,
    init("/bin/sh", args, options);
 }
 
+// Initialize for an interactive terminal
+void ChildProcess::init(const ProcessOptions& options)
+{
+   if (!options.stdOutFile.empty() || !options.stdErrFile.empty())
+   {
+      LOG_ERROR_MESSAGE(
+               "stdOutFile/stdErrFile options cannot be used with interactive terminal");
+   }
+
+   options_ = options;
+   exe_ = options_.shellPath.absolutePath();
+   args_ = options_.args;
+}
+
 ChildProcess::~ChildProcess()
 {
 }
@@ -302,6 +330,11 @@ Error ChildProcess::ptyInterrupt()
                          ERROR_LOCATION);
 }
 
+PidType ChildProcess::getPid()
+{
+   return pImpl_->pid;
+}
+
 Error ChildProcess::terminate()
 {
    // only send signal if the process is open
@@ -311,11 +344,20 @@ Error ChildProcess::terminate()
    // special code path for pseudoterminal
    if (options_.pseudoterminal)
    {
-      // on OSX you need to close all of the terminal handles to get
-      // bash to quit, however some other processes (like svn+ssh
-      // require the signal)
-#ifdef __APPLE__
-      pImpl_->closeAll(false, ERROR_LOCATION);
+#ifndef __APPLE__
+      // On Linux only do this if dealing with a Terminal-pane process.
+      // This is to reduce scope of this change for 1.1
+      // TODO: review post 1.1
+      if (options().smartTerminal)
+      {
+#endif
+         // you need to close all of the terminal handles to get
+         // bash to quit, however some other processes (like svn+ssh
+         // require the signal)
+         pImpl_->closeAll(false, ERROR_LOCATION);
+
+#ifndef __APPLE__
+      }
 #endif
 
       if (::killpg(::getpgid(pImpl_->pid), SIGTERM) == -1)
@@ -331,7 +373,7 @@ Error ChildProcess::terminate()
    else
    {
       // determine target pid (kill just this pid or pid + children)
-      pid_t pid = pImpl_->pid;
+      PidType pid = pImpl_->pid;
       if (options_.detachSession || options_.terminateChildren)
       {
          pid = -pid;
@@ -356,11 +398,34 @@ Error ChildProcess::terminate()
    }
 }
 
+bool ChildProcess::hasNonWhitelistSubprocess() const
+{
+   // base class doesn't support subprocess-checking; override to implement
+   return true;
+}
+
+bool ChildProcess::hasWhitelistSubprocess() const
+{
+   // base class doesn't support subprocess-checking; override to implement
+   return false;
+}
+
+core::FilePath ChildProcess::getCwd() const
+{
+   // base class doesn't support cwd-tracking; override to implement
+   return FilePath();
+}
+
+bool ChildProcess::hasRecentOutput() const
+{
+   // base class doesn't support output tracking; override to implement
+   return true;
+}
 
 Error ChildProcess::run()
 {  
    // declarations
-   pid_t pid = 0;
+   PidType pid = 0;
    int fdInput[2] = {0,0};
    int fdOutput[2] = {0,0};
    int fdError[2] = {0,0};
@@ -376,7 +441,7 @@ Error ChildProcess::run()
       winSize.ws_row = options_.pseudoterminal.get().rows;
       winSize.ws_xpixel = 0;
       winSize.ws_ypixel = 0;
-      Error error = posixCall<pid_t>(
+      Error error = posixCall<PidType>(
          boost::bind(::forkpty, &fdMaster, nullName, nullTermp, &winSize),
          ERROR_LOCATION,
          &pid);
@@ -410,7 +475,7 @@ Error ChildProcess::run()
       }
 
       // fork
-      error = posixCall<pid_t>(::fork, ERROR_LOCATION, &pid);
+      error = posixCall<PidType>(::fork, ERROR_LOCATION, &pid);
       if (error)
       {
          closePipe(fdInput, ERROR_LOCATION);
@@ -486,9 +551,26 @@ Error ChildProcess::run()
             ERROR_LOCATION);
          if (!error)
          {
-            // specify raw mode (but don't ignore signals -- this is done
-            // so we can send Ctrl-C for interrupts)
-            ::cfmakeraw(&termp);
+            if (!options_.smartTerminal)
+            {
+               // Specify raw mode; not doing this for terminal (versus dumb
+               // shell) because on Linux, it broke things like "passwd"
+               // command's ability to collect passwords).
+               ::cfmakeraw(&termp);
+            }
+            else
+            {
+               // for smart terminals we need to echo back the user input
+               termp.c_lflag |= ECHO;
+               termp.c_oflag |= OPOST|ONLCR;
+
+               // Turn off XON/XOFF flow control so Ctrl+S can be used by
+               // the shell command-line editing instead of suspending output.
+               termp.c_iflag &= ~(IXON|IXOFF);
+            }
+
+            // Don't ignore signals -- this is done
+            // so we can send Ctrl-C for interrupts
             termp.c_lflag |= ISIG;
 
             // set attribs
@@ -538,7 +620,10 @@ Error ChildProcess::run()
       {
          if (::chdir(options_.workingDir.absolutePath().c_str()))
          {
-            LOG_ERROR(systemError(errno, "Error changing directory", ERROR_LOCATION));
+            std::string message = "Error changing directory: '";
+            message += options_.workingDir.absolutePath().c_str();
+            message += "'";
+            LOG_ERROR(systemError(errno, message.c_str(), ERROR_LOCATION));
          }
       }
 
@@ -624,7 +709,7 @@ Error SyncChildProcess::waitForExit(int* pExitStatus)
 {
    // blocking wait for exit
    int status;
-   pid_t result = posixCall<pid_t>(
+   PidType result = posixCall<PidType>(
       boost::bind(::waitpid, pImpl_->pid, &status, 0));
 
    // always close all of the pipes
@@ -656,11 +741,11 @@ struct AsyncChildProcess::AsyncImpl
         exited_(false)
    {
    }
-
    bool calledOnStarted_;
    bool finishedStdout_;
    bool finishedStderr_;
    bool exited_;
+   boost::scoped_ptr<ChildProcessSubprocPoll> pSubprocPoll_;
 };
 
 AsyncChildProcess::AsyncChildProcess(const std::string& exe,
@@ -671,7 +756,7 @@ AsyncChildProcess::AsyncChildProcess(const std::string& exe,
    init(exe, args, options);
    if (!options.stdOutFile.empty() || !options.stdErrFile.empty())
    {
-      LOG_ERROR_MESSAGE(
+      LOG_WARNING_MESSAGE(
                "stdOutFile/stdErrFile options cannot be used with runProgram");
    }
 }
@@ -683,23 +768,67 @@ AsyncChildProcess::AsyncChildProcess(const std::string& command,
    init(command, options);
 }
 
+AsyncChildProcess::AsyncChildProcess(const ProcessOptions& options)
+      : ChildProcess(), pAsyncImpl_(new AsyncImpl())
+{
+   init(options);
+}
+
 AsyncChildProcess::~AsyncChildProcess()
 {
 }
 
 Error AsyncChildProcess::terminate()
 {
-#ifdef __APPLE__
-   if (options().pseudoterminal)
+#ifndef __APPLE__
+   // On Linux only do this if dealing with a Terminal-pane process.
+   // This is to reduce scope of this change for 1.1
+   // TODO: review post 1.1
+   if (options().smartTerminal)
    {
-      pAsyncImpl_->finishedStderr_ = true;
-      pAsyncImpl_->finishedStdout_ = true;
+#endif
+      if (options().pseudoterminal)
+      {
+         pAsyncImpl_->finishedStderr_ = true;
+         pAsyncImpl_->finishedStdout_ = true;
+      }
+#ifndef __APPLE__
    }
 #endif
-
    return ChildProcess::terminate();
 }
 
+bool AsyncChildProcess::hasNonWhitelistSubprocess() const
+{
+   if (pAsyncImpl_->pSubprocPoll_)
+      return pAsyncImpl_->pSubprocPoll_->hasNonWhitelistSubprocess();
+   else
+      return true;
+}
+
+bool AsyncChildProcess::hasWhitelistSubprocess() const
+{
+   if (pAsyncImpl_->pSubprocPoll_)
+      return pAsyncImpl_->pSubprocPoll_->hasWhitelistSubprocess();
+   else
+      return false;
+}
+
+core::FilePath AsyncChildProcess::getCwd() const
+{
+   if (pAsyncImpl_->pSubprocPoll_)
+      return pAsyncImpl_->pSubprocPoll_->getCwd();
+   else
+      return FilePath();
+}
+
+bool AsyncChildProcess::hasRecentOutput() const
+{
+   if (pAsyncImpl_->pSubprocPoll_)
+      return pAsyncImpl_->pSubprocPoll_->hasRecentOutput();
+   else
+      return true;
+}
 
 void AsyncChildProcess::poll()
 {
@@ -716,11 +845,18 @@ void AsyncChildProcess::poll()
       else
          setPipeNonBlocking(pImpl_->fdStderr);         
 
+      // setup for subprocess polling
+      pAsyncImpl_->pSubprocPoll_.reset(new ChildProcessSubprocPoll(
+         pImpl_->pid,
+         kResetRecentDelay, kCheckSubprocDelay, kCheckCwdDelay,
+         options().reportHasSubprocs ? core::system::getSubprocesses : NULL,
+         options().subprocWhitelist,
+         options().trackCwd ? core::system::currentWorkingDir : NULL));
+
       if (callbacks_.onStarted)
          callbacks_.onStarted(*this);
       pAsyncImpl_->calledOnStarted_ = true;
    }
-
    // call onContinue
    if (callbacks_.onContinue)
    {
@@ -733,6 +869,8 @@ void AsyncChildProcess::poll()
       }
    }
 
+   bool hasRecentOutput = false;
+
    // check stdout and fire event if we got output
    if (!pAsyncImpl_->finishedStdout_)
    {
@@ -746,7 +884,10 @@ void AsyncChildProcess::poll()
       else
       {
          if (!out.empty() && callbacks_.onStdout)
+         {
+            hasRecentOutput = true;
             callbacks_.onStdout(*this, out);
+         }
 
          if (eof)
            pAsyncImpl_->finishedStdout_ = true;
@@ -767,14 +908,16 @@ void AsyncChildProcess::poll()
       else
       {
          if (!err.empty() && callbacks_.onStderr)
+         {
+            hasRecentOutput = true;
             callbacks_.onStderr(*this, err);
+         }
 
          if (eof)
            pAsyncImpl_->finishedStderr_ = true;
       }
    }
 
-
    // Check for exited. Note that this method specifies WNOHANG
    // so we don't block forever waiting for a process the exit. We may
    // not be able to reap the child due to an error (typically ECHILD,
@@ -782,7 +925,7 @@ void AsyncChildProcess::poll()
    // case we'll allow the exit sequence to proceed and simply pass -1 as
    // the exit status.
    int status;
-   pid_t result = posixCall<pid_t>(
+   PidType result = posixCall<PidType>(
             boost::bind(::waitpid, pImpl_->pid, &status, WNOHANG));
 
    // either a normal exit or an error while waiting
@@ -807,6 +950,7 @@ void AsyncChildProcess::poll()
       // set exited_ flag so that our exited function always
       // returns the right value
       pAsyncImpl_->exited_ = true;
+      pAsyncImpl_->pSubprocPoll_->stop();
 
       // if this is an error that isn't ECHILD then log it (we never
       // expect this to occur as the only documented error codes are
@@ -814,6 +958,20 @@ void AsyncChildProcess::poll()
       if (result == -1 && errno != ECHILD && errno != ENOENT)
          LOG_ERROR(systemError(errno, ERROR_LOCATION));
    }
+
+   // Perform optional periodic operations
+   if (pAsyncImpl_->pSubprocPoll_->poll(hasRecentOutput))
+   {
+      if (callbacks_.onHasSubprocs)
+      {
+         callbacks_.onHasSubprocs(hasNonWhitelistSubprocess(),
+                                  hasWhitelistSubprocess());
+      }
+      if (callbacks_.reportCwd)
+      {
+         callbacks_.reportCwd(getCwd());
+      }
+   }
 }
 
 bool AsyncChildProcess::exited()
