@@ -23,6 +23,7 @@
 
 #include <boost/foreach.hpp>
 
+#include <QClipboard>
 #include <QFileDialog>
 #include <QMessageBox>
 #include <QApplication>
@@ -57,8 +58,9 @@ namespace rstudio {
 namespace desktop {
 
 namespace {
-   WindowTracker s_windowTracker;
-}
+WindowTracker s_windowTracker;
+QString s_globalMouseSelection;
+} // end anonymous namespace
 
 extern QString scratchPath;
 
@@ -68,6 +70,15 @@ GwtCallback::GwtCallback(MainWindow* pMainWindow, GwtCallbackOwner* pOwner)
      pSynctex_(NULL),
      pendingQuit_(PendingQuitNone)
 {
+    QClipboard* clipboard = QApplication::clipboard();
+
+    // listen for clipboard data change events
+    QObject::connect(clipboard, SIGNAL(changed(QClipboard::Mode)),
+                     this, SLOT(onClipboardChanged(QClipboard::Mode)));
+
+    // initialize the global selection
+    if (clipboard->supportsSelection())
+        s_globalMouseSelection = clipboard->text(QClipboard::Selection);
 }
 
 Synctex& GwtCallback::synctex()
@@ -143,22 +154,54 @@ QString resolveAliasedPath(const QString& path)
 } // anonymous namespace
 
 QString GwtCallback::getOpenFileName(const QString& caption,
-                                    const QString& dir,
-                                    const QString& filter)
+                                     const QString& label,
+                                     const QString& dir,
+                                     const QString& filter)
 {
    QString resolvedDir = resolveAliasedPath(dir);
-   QString result = QFileDialog::getOpenFileName(pOwner_->asWidget(),
-                                                 caption,
-                                                 resolvedDir,
-                                                 filter,
-                                                 0,
-                                                 standardFileDialogOptions());
+
+   QFileDialog dialog(
+            pOwner_->asWidget(),
+            caption,
+            resolveAliasedPath(dir),
+            filter);
+
+   dialog.setFileMode(QFileDialog::ExistingFile);
+   dialog.setLabelText(QFileDialog::Accept, label);
+
+   QString result;
+   if (dialog.exec() == QDialog::Accepted)
+      result = dialog.selectedFiles().value(0);
 
    activateAndFocusOwner();
    return createAliasedPath(result);
 }
 
+namespace {
+
+QString getSaveFileNameImpl(QWidget* pParent,
+                            const QString& caption,
+                            const QString& label,
+                            const QString& dir,
+                            QFileDialog::Options options)
+{
+    // initialize dialog
+    QFileDialog dialog(pParent, caption, dir);
+    dialog.setOptions(options);
+    dialog.setLabelText(QFileDialog::Accept, label);
+    dialog.setAcceptMode(QFileDialog::AcceptSave);
+
+    // execute dialog and check for success
+    if (dialog.exec() == QDialog::Accepted)
+        return dialog.selectedFiles().value(0);
+
+    return QString();
+}
+
+} // end anonymous namespace
+
 QString GwtCallback::getSaveFileName(const QString& caption,
+                                     const QString& label,
                                      const QString& dir,
                                      const QString& defaultExtension,
                                      bool forceDefaultExtension)
@@ -167,8 +210,13 @@ QString GwtCallback::getSaveFileName(const QString& caption,
 
    while (true)
    {
-      QString result = QFileDialog::getSaveFileName(pOwner_->asWidget(), caption, resolvedDir,
-                                                    QString(), 0, standardFileDialogOptions());
+      QString result = getSaveFileNameImpl(
+                  pOwner_->asWidget(),
+                  caption,
+                  label,
+                  resolvedDir,
+                  standardFileDialogOptions());
+
       activateAndFocusOwner();
       if (result.isEmpty())
          return result;
@@ -205,11 +253,12 @@ QString GwtCallback::getSaveFileName(const QString& caption,
 }
 
 QString GwtCallback::getExistingDirectory(const QString& caption,
-                                         const QString& dir)
+                                          const QString& label,
+                                          const QString& dir)
 {
-   QString resolvedDir = resolveAliasedPath(dir);
-
    QString result;
+
+   // TODO: can we remove this code below?
 #ifdef _WIN32
    if (dir.isNull())
    {
@@ -229,21 +278,49 @@ QString GwtCallback::getExistingDirectory(const QString& caption,
          result = QString();
       else
          result = QString::fromWCharArray(szDir);
+      activateAndFocusOwner();
+      return createAliasedPath(result);
    }
-   else
-   {
-      result = QFileDialog::getExistingDirectory(pOwner_->asWidget(), caption, resolvedDir,
-                                                 QFileDialog::ShowDirsOnly | standardFileDialogOptions());
-   }
-#else
-   result = QFileDialog::getExistingDirectory(pOwner_->asWidget(), caption, resolvedDir,
-                                              QFileDialog::ShowDirsOnly | standardFileDialogOptions());
 #endif
 
+   QFileDialog dialog(
+            pOwner_->asWidget(),
+            caption,
+            resolveAliasedPath(dir));
+
+   dialog.setLabelText(QFileDialog::Accept, label);
+   dialog.setFileMode(QFileDialog::Directory);
+   dialog.setOption(QFileDialog::ShowDirsOnly, true);
+   if (dialog.exec() == QDialog::Accepted)
+      result = dialog.selectedFiles().value(0);
+
    activateAndFocusOwner();
    return createAliasedPath(result);
 }
 
+void GwtCallback::onClipboardChanged(QClipboard::Mode mode)
+{
+    // if this is a change in the selection contents, track it
+    if (mode == QClipboard::Selection)
+    {
+        QClipboard* clipboard = QApplication::clipboard();
+        QString text = clipboard->text(mode);
+
+        // when one clicks on an Ace instance, a hidden length-one selection
+        // will sneak in here. explicitly screen those out.
+        if (text == QString::fromUtf8("\x01"))
+        {
+           // restore the old global selection
+           clipboard->setText(s_globalMouseSelection, QClipboard::Selection);
+        }
+        else
+        {
+           // otherwise, update our tracked global selection
+           s_globalMouseSelection = text;
+        }
+    }
+}
+
 void GwtCallback::doAction(const QKeySequence& keys)
 {
    int keyCode = keys[0];
@@ -295,6 +372,31 @@ void GwtCallback::clipboardPaste()
    doAction(QKeySequence::Paste);
 }
 
+void GwtCallback::setClipboardText(QString text)
+{
+   QClipboard* pClipboard = QApplication::clipboard();
+   pClipboard->setText(text, QClipboard::Clipboard);
+}
+
+QString GwtCallback::getClipboardText()
+{
+   QClipboard* pClipboard = QApplication::clipboard();
+   return pClipboard->text(QClipboard::Clipboard);
+}
+
+void GwtCallback::setGlobalMouseSelection(QString selection)
+{
+    QClipboard* clipboard = QApplication::clipboard();
+    if (clipboard->supportsSelection())
+        clipboard->setText(selection, QClipboard::Selection);
+    s_globalMouseSelection = selection;
+}
+
+QString GwtCallback::getGlobalMouseSelection()
+{
+    return s_globalMouseSelection;
+}
+
 QString GwtCallback::proportionalFont()
 {
    return options().proportionalFont();
