@@ -235,6 +235,29 @@ SEXP asNamespace(const std::string& name)
    return findNamespace(name);
 }
 
+SEXP forcePromise(SEXP objectSEXP)
+{
+   // if this isn't a promise, return it as-is
+   if (TYPEOF(objectSEXP) != PROMSXP)
+      return objectSEXP;
+   
+   // if we already have a forced value, return that
+   SEXP valueSEXP = PRVALUE(objectSEXP);
+   if (valueSEXP != R_UnboundValue)
+      return valueSEXP;
+   
+   // otherwise, evaluate the promise and return that result
+   r::sexp::Protect protect;
+   SEXP resultSEXP;
+   protect.add(resultSEXP = ::Rf_eval(PRCODE(objectSEXP), PRENV(objectSEXP)));
+   
+   // update the promise reference
+   SET_PRVALUE(objectSEXP, resultSEXP);
+   
+   // return the result
+   return resultSEXP;
+}
+
 SEXP findNamespace(const std::string& name)
 {
    if (name.empty())
@@ -304,6 +327,110 @@ void listEnvironment(SEXP env,
    }
 }
 
+namespace {
+
+Error asPrimitiveEnvironment(SEXP envirSEXP,
+                             SEXP* pTargetSEXP,
+                             Protect* pProtect)
+{
+   // fast-case: no need to call back into R
+   if (TYPEOF(envirSEXP) == ENVSXP)
+   {
+      pProtect->add(*pTargetSEXP = envirSEXP);
+      return Success();
+   }
+   
+   // for non-S4 objects, we can just return an error (false) early
+   if (TYPEOF(envirSEXP) != S4SXP)
+      return Error(errc::UnexpectedDataTypeError, ERROR_LOCATION);
+   
+   // use R function to convert
+   Error error = RFunction("base:::as.environment")
+         .addParam(envirSEXP)
+         .call(pTargetSEXP, pProtect);
+   
+   if (error)
+      return error;
+   
+   // ensure that we actually succeeded in producing a primitive environment
+   if (pTargetSEXP == NULL  ||
+       *pTargetSEXP == NULL ||
+       !isPrimitiveEnvironment(*pTargetSEXP))
+   {
+      return Error(errc::UnexpectedDataTypeError, ERROR_LOCATION);
+   }
+   
+   // we have a primitive environment; all is well
+   return Success();
+}
+
+bool hasActiveBindingImpl(const std::string& name,
+                          SEXP envirSEXP,
+                          std::set<SEXP>* pVisitedObjects)
+{
+   Error error;
+   Protect protect;
+   
+   // ensure we have an environment
+   if (!isEnvironment(envirSEXP))
+      return false;
+   
+   // sanity check that we are working with a primitive environment
+   // (required to convert S4 objects that subclass 'environment' into
+   // a 'raw' R environment object)
+   error = asPrimitiveEnvironment(envirSEXP, &envirSEXP, &protect);
+   if (error)
+      return false;
+   
+   // check for active binding
+   if (isActiveBinding(name, envirSEXP))
+      return true;
+   
+   // resolve the object (discover in that frame)
+   SEXP nameSEXP = Rf_install(name.c_str());
+   SEXP varSEXP = Rf_findVarInFrame(envirSEXP, nameSEXP);
+   
+   // check for special values
+   if (varSEXP == R_UnboundValue || varSEXP == R_MissingArg)
+      return false;
+   
+   // ensure we're working with a primitive R environment
+   if (!isEnvironment(varSEXP))
+      return false;
+   
+   error = asPrimitiveEnvironment(varSEXP, &varSEXP, &protect);
+   if (error)
+      return false;
+   
+   // avoid cycles
+   if (pVisitedObjects->count(varSEXP)) return false;
+   pVisitedObjects->insert(varSEXP);
+   
+   // list the bindings in this object
+   SEXP bindingsSEXP;
+   protect.add(bindingsSEXP = R_lsInternal(varSEXP, TRUE));
+   
+   // iterate over items and search for active bindings
+   for (int i = 0, n = Rf_length(bindingsSEXP); i < n; ++i)
+   {
+      const char* binding = CHAR(STRING_ELT(bindingsSEXP, i));
+      if (hasActiveBindingImpl(binding, varSEXP, pVisitedObjects))
+         return true;
+   }
+   
+   // no child binding has active binding; return false
+   return false;
+}
+
+} // end anonymous namespace
+
+bool hasActiveBinding(const std::string& name, const SEXP envirSEXP)
+{
+   // avoid cycles when searching recursively
+   std::set<SEXP> visitedObjects;
+   return hasActiveBindingImpl(name, envirSEXP, &visitedObjects);
+}
+
 bool isActiveBinding(const std::string& name, const SEXP env)
 {
    // R_BindingIsActive throws error on .Last.value check; avoid that and
@@ -457,6 +584,33 @@ bool isNull(SEXP object)
    return Rf_isNull(object) == TRUE;
 }
 
+bool isPrimitiveEnvironment(SEXP object)
+{
+   return TYPEOF(object) == ENVSXP;
+}
+
+bool isEnvironment(SEXP object)
+{
+   // detect primitive environments (fast path)
+   if (isPrimitiveEnvironment(object))
+      return true;
+   
+   // call back to R to detect objects subclassing environment
+   if (TYPEOF(object) == S4SXP)
+   {
+      bool result = false;
+      Error error = RFunction("base:::is.environment")
+            .addParam(object)
+            .call(&result);
+
+      if (error)
+         LOG_ERROR(error);
+      
+      return result;
+   }
+   
+   return false;
+}
 
 SEXP getNames(SEXP sexp)
 {
@@ -512,6 +666,18 @@ SEXP setAttrib(SEXP object, const std::string& attrib, SEXP val)
    return Rf_setAttrib(object, Rf_install(attrib.c_str()), val);
 }
 
+bool isExternalPointer(SEXP object)
+{
+   return TYPEOF(object) == EXTPTRSXP;
+}
+
+bool isNullExternalPointer(SEXP object)
+{
+   return
+         isExternalPointer(object) &&
+         R_ExternalPtrAddr(object) == NULL;
+}
+
 SEXP makeWeakRef(SEXP key, SEXP val, R_CFinalizer_t fun, Rboolean onexit)
 {
    return R_MakeWeakRefC(key, val, fun, onexit);
