@@ -173,6 +173,7 @@ void httpServerAddHandlers()
    uri_handlers::add("/presentation", secureAsyncHttpHandler(proxyContentRequest));
    uri_handlers::add("/pdf_js", secureAsyncHttpHandler(proxyContentRequest));
    uri_handlers::add("/mathjax", secureAsyncHttpHandler(proxyContentRequest));
+   uri_handlers::add("/connections", secureAsyncHttpHandler(proxyContentRequest));
 
    // content handlers which might be accessed outside the context of the
    // workbench get secure + authentication when required
@@ -188,6 +189,7 @@ void httpServerAddHandlers()
    uri_handlers::add("/chunk_output", secureAsyncHttpHandler(proxyContentRequest, true)); 
    uri_handlers::add("/profiles", secureAsyncHttpHandler(proxyContentRequest, true));
    uri_handlers::add("/rmd_data", secureAsyncHttpHandler(proxyContentRequest, true));
+   uri_handlers::add("/profiler_resource", secureAsyncHttpHandler(proxyContentRequest, true));
 
    // proxy localhost if requested
    if (server::options().wwwProxyLocalhost())
