@@ -1,7 +1,7 @@
 /*
  * SessionModuleContext.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2009-16 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -62,6 +62,7 @@
 #include <r/session/RConsoleActions.hpp>
 
 #include <session/SessionOptions.hpp>
+#include <session/SessionPersistentState.hpp>
 #include "SessionClientEventQueue.hpp"
 
 #include <session/projects/SessionProjects.hpp>
@@ -147,6 +148,30 @@ SEXP rs_enqueClientEvent(SEXP nameSEXP, SEXP dataSEXP)
         type = session::client_events::kRprofCreated;
       else if (name == "editor_command")
          type = session::client_events::kEditorCommand;
+      else if (name == "navigate_shiny_frame")
+         type = session::client_events::kNavigateShinyFrame;
+      else if (name == "update_new_connection_dialog")
+         type = session::client_events::kUpdateNewConnectionDialog;
+      else if (name == "terminal_subprocs")
+         type = session::client_events::kTerminalSubprocs;
+      else if (name == "rstudioapi_show_dialog")
+         type = session::client_events::kRStudioAPIShowDialog;
+      else if (name == "object_explorer_event")
+         type = session::client_events::kObjectExplorerEvent;
+      else if (name == "send_to_terminal")
+         type = session::client_events::kSendToTerminal;
+      else if (name == "clear_terminal")
+         type = session::client_events::kClearTerminal;
+      else if (name == "add_terminal")
+         type = session::client_events::kAddTerminal;
+      else if (name == "activate_terminal")
+         type = session::client_events::kActivateTerminal;
+      else if (name == "terminal_cwd")
+         type = session::client_events::kTerminalCwd;
+      else if (name == "remove_terminal")
+         type = session::client_events::kRemoveTerminal;
+      else if (name == "show_page_viewer")
+         type = session::client_events::kShowPageViewerEvent;
 
       if (type != -1)
       {
@@ -366,6 +391,34 @@ SEXP rs_markdownToHTML(SEXP contentSEXP)
    return r::sexp::create(htmlContent, &rProtect);
 }
 
+inline std::string persistantValueName(SEXP nameSEXP)
+{
+   return "rstudioapi_persistent_values_" + r::sexp::safeAsString(nameSEXP);
+}
+
+SEXP rs_setPersistentValue(SEXP nameSEXP, SEXP valueSEXP)
+{
+   std::string name = persistantValueName(nameSEXP);
+   std::string value = r::sexp::safeAsString(valueSEXP);
+   persistentState().settings().set(name, value);
+   return R_NilValue;
+}
+
+SEXP rs_getPersistentValue(SEXP nameSEXP)
+{
+   std::string name = persistantValueName(nameSEXP);
+   if (persistentState().settings().contains(name))
+   {
+      std::string value = persistentState().settings().get(name);
+      r::sexp::Protect rProtect;
+      return r::sexp::create(value, &rProtect);
+   }
+   else
+   {
+      return R_NilValue;
+   }
+}
+
 } // anonymous namespace
 
 
@@ -691,6 +744,24 @@ bool isPackagePosixMakefile(const FilePath& srcPath)
            filename == "Makefile.in");
 }
 
+void performIdleOnlyAsyncRpcMethod(
+      const core::json::JsonRpcRequest& request,
+      const core::json::JsonRpcFunctionContinuation& continuation,
+      const core::json::JsonRpcAsyncFunction& function)
+{
+   if (request.isBackgroundConnection)
+   {
+      module_context::scheduleDelayedWork(
+          boost::posix_time::milliseconds(100),
+          boost::bind(function, request, continuation),
+          true);
+   }
+   else
+   {
+      function(request, continuation);
+   }
+}
+
 } // anonymous namespeace
 
 void scheduleDelayedWork(const boost::posix_time::time_duration& period,
@@ -723,6 +794,16 @@ void onBackgroundProcessing(bool isIdle)
       executeScheduledCommands(&s_idleScheduledCommands);
 }
 
+
+Error registerIdleOnlyAsyncRpcMethod(
+                             const std::string& name,
+                             const core::json::JsonRpcAsyncFunction& function)
+{
+   return registerAsyncRpcMethod(name,
+                                 boost::bind(performIdleOnlyAsyncRpcMethod,
+                                                _1, _2, function));
+}
+
 core::string_utils::LineEnding lineEndings(const core::FilePath& srcFile)
 {
    // potential special case for Makevars
@@ -1056,7 +1137,7 @@ std::vector<FilePath> getLibPaths()
    if (error)
       LOG_ERROR(error);
 
-   std::vector<FilePath> libPaths(libPathsString.size());
+   std::vector<FilePath> libPaths;
    BOOST_FOREACH(const std::string& path, libPathsString)
    {
       libPaths.push_back(module_context::resolveAliasedPath(path));
@@ -1324,6 +1405,20 @@ SEXP rs_base64encode(SEXP dataSEXP, SEXP binarySEXP)
       return r::sexp::create(output, &protect);
 }
 
+SEXP rs_base64encodeFile(SEXP pathSEXP)
+{
+   std::string path = r::sexp::asString(pathSEXP); 
+   FilePath filePath = module_context::resolveAliasedPath(path);
+
+   std::string output;
+   Error error = base64::encode(filePath, &output);
+   if (error)
+      LOG_ERROR(error);
+
+   r::sexp::Protect protect;
+   return r::sexp::create(output, &protect);
+}
+
 SEXP rs_base64decode(SEXP dataSEXP, SEXP binarySEXP)
 {
    bool binary = r::sexp::asLogical(binarySEXP);
@@ -1426,8 +1521,45 @@ r_util::ActiveSession& activeSession()
       std::string id = options().sessionScope().id();
       if (!id.empty())
          pSession = activeSessions().get(id);
+      else if (options().programMode() == kSessionProgramModeDesktop)
+      {
+         // if no active session, create one and use the launcher token as a
+         // synthetic session ID
+         //
+         // we only do this in desktop mode to preserve backwards compatibility
+         // with some functionality that depends on session data
+         // persisting after rstudio has been closed
+         //
+         // this entire clause will likely need to be reverted in a future release
+         // once we ensure that all execution modes have this squared away
+         pSession = activeSessions().emptySession(options().launcherToken());
+      }
       else
-         pSession = activeSessions().emptySession();
+      {
+         // if no scope was specified, we are in singleton session mode
+         // check to see if there is an existing active session, and use that
+         std::vector<boost::shared_ptr<r_util::ActiveSession> > sessions =
+               activeSessions().list(userHomePath(), options().projectSharingEnabled());
+         if (sessions.size() == 1)
+         {
+            // there is only one session, so this must be singleton session mode
+            // reopen that session
+            pSession = sessions.front();
+         }
+         else
+         {
+            // create a new session entry
+            // this should not really run because in single session mode there will
+            // only be one session but we'll create one here just in case for safety
+            std::string sessionId;
+
+            // pass ~ as the default working directory
+            // this is resolved deeper in the code in SessionClientInit to turn into
+            // the actual user preference or session default directory that it should be
+            activeSessions().create(options().sessionScope().project(), "~", &sessionId);
+            pSession = activeSessions().get(sessionId);
+         }
+      }
    }
    return *pSession;
 }
@@ -1779,7 +1911,7 @@ bool portmapPathForLocalhostUrl(const std::string& url, std::string* pPath)
    // extract the port
    boost::regex re("http[s]?://(?:localhost|127\\.0\\.0\\.1):([0-9]+)(/.*)?");
    boost::smatch match;
-   if (boost::regex_search(url, match, re))
+   if (regex_utils::search(url, match, re))
    {
       // calculate the path
       std::string path = match[2];
@@ -2004,6 +2136,24 @@ core::Error recursiveCopyDirectory(const core::FilePath& fromDir,
    return fileCopy.call();
 }
 
+bool isSessionTempPath(FilePath filePath)
+{
+   // get the real path
+   Error error = core::system::realPath(filePath, &filePath);
+   if (error)
+      LOG_ERROR(error);
+
+   // get the session temp dir real path; needed since the file path above is
+   // also a real path--e.g. on OS X, it refers to /private/tmp rather than
+   // /tmp
+   FilePath tempDir;
+   error = core::system::realPath(module_context::tempDir(), &tempDir);
+   if (error)
+      LOG_ERROR(error);
+
+   return filePath.isWithin(tempDir);
+}
+
 std::string sessionTempDirUrl(const std::string& sessionTempPath)
 {
    if (session::options().programMode() == kSessionProgramModeDesktop)
@@ -2073,7 +2223,7 @@ json::Object plotExportFormat(const std::string& name,
 Error createSelfContainedHtml(const FilePath& sourceFilePath,
                               const FilePath& targetFilePath)
 {
-   r::exec::RFunction func("rmarkdown:::pandoc_self_contained_html");
+   r::exec::RFunction func(".rs.pandocSelfContainedHtml");
    func.addParam(string_utils::utf8ToSystem(sourceFilePath.absolutePath()));
    func.addParam(string_utils::utf8ToSystem(targetFilePath.absolutePath()));
    return func.call();
@@ -2303,6 +2453,10 @@ Error initialize()
    methodDef17.numArgs = 1;
    r::routines::addCallMethod(methodDef17);
    
+   // register persistent value functions
+   RS_REGISTER_CALL_METHOD(rs_setPersistentValue, 2);
+   RS_REGISTER_CALL_METHOD(rs_getPersistentValue, 1);
+
    // register rs_isRScriptInPackageBuildTarget
    r::routines::registerCallMethod(
             "rs_isRScriptInPackageBuildTarget",
@@ -2339,6 +2493,7 @@ Error initialize()
             0);
 
    RS_REGISTER_CALL_METHOD(rs_base64encode, 2);
+   RS_REGISTER_CALL_METHOD(rs_base64encodeFile, 1);
    RS_REGISTER_CALL_METHOD(rs_base64decode, 2);
 
    // initialize monitored scratch dir
