@@ -34,7 +34,7 @@ core::Error initialize();
                        
 } // namespace console
 } // namespace modules
-} // namesapce session
+} // namespace session
 } // namespace rstudio
 
 #endif // SESSION_CONSOLE_HPP
