@@ -33,6 +33,7 @@
 #include <core/FileSerializer.hpp>
 #include <core/StringUtils.hpp>
 #include <core/text/TemplateFilter.hpp>
+#include <core/r_util/RProjectFile.hpp>
 #include <core/r_util/RPackageInfo.hpp>
 
 #include <core/json/JsonRpc.hpp>
@@ -64,6 +65,8 @@ using namespace session::source_database;
 
 namespace {
 
+module_context::WaitForMethodFunction s_waitForRequestDocumentSave;
+
 void writeDocToJson(boost::shared_ptr<SourceDocument> pDoc,
                     core::json::Object* pDocJson)
 {
@@ -84,6 +87,35 @@ void writeDocToJson(boost::shared_ptr<SourceDocument> pDoc,
          LOG_ERROR(error);
    }
    (*pDocJson)["notebook"] = notebook;
+   
+   // discover project-specific settings when available
+   r_util::RProjectConfig projConfig;
+   FilePath docPath = module_context::resolveAliasedPath(pDoc->path());
+   FilePath projDir = projects::projectContext().directory();
+   bool hasConfig = false;
+   
+   if (docPath.isWithin(projDir))
+   {
+      projConfig = projects::projectContext().config();
+      hasConfig = true;
+   }
+   else
+   {
+      Error error = r_util::findProjectConfig(docPath, &projConfig);
+      hasConfig = !error;
+   }
+   
+   if (hasConfig)
+   {
+      json::Object projConfigJson;
+      
+      projConfigJson["tab_size"] = projConfig.numSpacesForTab;
+      projConfigJson["use_soft_tabs"] = projConfig.useSpacesForTab;
+      projConfigJson["strip_trailing_whitespace"] = projConfig.stripTrailingWhitespace;
+      projConfigJson["ensure_trailing_newline"] = projConfig.autoAppendNewline;
+      
+      (*pDocJson)["project_config"] = projConfigJson;
+   }
 }
 
 void detectExtendedType(boost::shared_ptr<SourceDocument> pDoc)
@@ -110,11 +142,11 @@ int numSourceDocuments()
 
 // wrap source_database::put for situations where there are new contents
 // (so we can index the contents)
-Error sourceDatabasePutWithUpdatedContents(
-                              boost::shared_ptr<SourceDocument> pDoc)
+Error sourceDatabasePutWithUpdatedContents(boost::shared_ptr<SourceDocument> pDoc,
+                                           bool writeContents = true)
 {
    // write the file to the database
-   Error error = source_database::put(pDoc);
+   Error error = source_database::put(pDoc, writeContents);
    if (error)
       return error ;
 
@@ -393,15 +425,17 @@ Error saveDocument(const json::JsonRpcRequest& request,
    boost::shared_ptr<SourceDocument> pDoc(new SourceDocument());
    error = source_database::get(id, pDoc);
    if (error)
-      return error ;
+      return error;
    
+   // check if the document contents have changed
+   bool hasChanges = contents != pDoc->contents();
    error = saveDocumentCore(contents, jsonPath, jsonType, jsonEncoding,
                             jsonFoldSpec, jsonChunkOutput, pDoc);
    if (error)
       return error;
    
    // write to the source_database
-   error = sourceDatabasePutWithUpdatedContents(pDoc);
+   error = sourceDatabasePutWithUpdatedContents(pDoc, hasChanges);
    if (error)
       return error;
 
@@ -476,13 +510,16 @@ Error saveDocumentDiff(const json::JsonRpcRequest& request,
       contents.erase(rangeBegin, rangeEnd);
       contents.insert(rangeBegin, replacement.begin(), replacement.end());
       
+      // track if we're updating the document contents
+      bool hasChanges = contents != pDoc->contents();
       error = saveDocumentCore(contents, jsonPath, jsonType, jsonEncoding,
                                jsonFoldSpec, jsonChunkOutput, pDoc);
       if (error)
          return error;
       
-      // write to the source_database
-      error = sourceDatabasePutWithUpdatedContents(pDoc);
+      // write to the source database (don't worry about writing document
+      // contents if those have not changed)
+      error = sourceDatabasePutWithUpdatedContents(pDoc, hasChanges);
       if (error)
          return error;
 
@@ -666,13 +703,13 @@ Error modifyDocumentProperties(const json::JsonRpcRequest& request,
 
    // get the doc
    boost::shared_ptr<SourceDocument> pDoc(new SourceDocument());
-   error = source_database::get(id, pDoc);
+   error = source_database::get(id, false, pDoc);
    if (error)
       return error ;
 
    // edit properties and write the document
    pDoc->editProperties(properties);
-   return source_database::put(pDoc);
+   return source_database::put(pDoc, false);
 }
 
 Error getDocumentProperties(const json::JsonRpcRequest& request,
@@ -1163,6 +1200,33 @@ SEXP rs_fileEdit(SEXP fileSEXP)
    return R_NilValue;
 }
 
+SEXP rs_requestDocumentSave(SEXP idsSEXP)
+{
+   r::sexp::Protect protect;
+   
+   json::Object jsonData;
+   
+   jsonData["ids"] = json::Value();
+   if (TYPEOF(idsSEXP) == STRSXP)
+   {
+      std::vector<std::string> ids;
+      r::sexp::fillVectorString(idsSEXP, &ids);
+      jsonData["ids"] = json::toJsonArray(ids);
+   }
+   
+   json::JsonRpcRequest request;
+   ClientEvent event(client_events::kRequestDocumentSave, jsonData);
+   if (!s_waitForRequestDocumentSave(&request, event))
+      return r::sexp::create(false, &protect);
+   
+   bool success = false;
+   Error error = json::readParams(request.params, &success);
+   if (error)
+      LOG_ERROR(error);
+   
+   return r::sexp::create(success, &protect);
+}
+
 } // anonymous namespace
 
 Error clientInitDocuments(core::json::Array* pJsonDocs)
@@ -1213,13 +1277,13 @@ Error initialize()
 
    // add suspend/resume handler
    addSuspendHandler(SuspendHandler(boost::bind(onSuspend, _2), onResume));
+   
+   // register waitfor methods
+   s_waitForRequestDocumentSave =
+         module_context::registerWaitForMethod("request_document_save_completed");
 
-   // register fileEdit method
-   R_CallMethodDef methodDef ;
-   methodDef.name = "rs_fileEdit" ;
-   methodDef.fun = (DL_FUNC) rs_fileEdit ;
-   methodDef.numArgs = 1;
-   r::routines::addCallMethod(methodDef);
+   RS_REGISTER_CALL_METHOD(rs_fileEdit, 1);
+   RS_REGISTER_CALL_METHOD(rs_requestDocumentSave, 1);
 
    // install rpc methods
    using boost::bind;
@@ -1262,6 +1326,6 @@ Error initialize()
 
 } // namespace source
 } // namespace modules
-} // namesapce session
+} // namespace session
 } // namespace rstudio
 
