@@ -1,7 +1,7 @@
 /*
  * SessionWorkbench.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2009-17 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -13,7 +13,6 @@
  *
  */
 
-
 #include "SessionWorkbench.hpp"
 
 #include <algorithm>
@@ -32,7 +31,6 @@
 
 #include <core/json/JsonRpc.hpp>
 
-#include <core/system/Environment.hpp>
 #include <core/system/ShellUtils.hpp>
 
 #include <r/ROptions.hpp>
@@ -47,10 +45,7 @@
 #include <session/SessionUserSettings.hpp>
 #include <session/SessionConsoleProcess.hpp>
 #include <session/RVersionSettings.hpp>
-
-#include "SessionVCS.hpp"
-#include "SessionGit.hpp"
-#include "SessionSVN.hpp"
+#include <session/SessionTerminalShell.hpp>
 
 #include "SessionSpelling.hpp"
 
@@ -195,6 +190,7 @@ Error setWorkbenchMetrics(const json::JsonRpcRequest& request,
    r::session::RClientMetrics metrics ;
    Error error = json::readObjectParam(request.params, 0,
                                  "consoleWidth", &(metrics.consoleWidth),
+                                 "buildConsoleWidth", &(metrics.buildConsoleWidth),
                                  "graphicsWidth", &(metrics.graphicsWidth),
                                  "graphicsHeight", &(metrics.graphicsHeight),
                                  "devicePixelRatio", &(metrics.devicePixelRatio));
@@ -262,7 +258,8 @@ Error setPrefs(const json::JsonRpcRequest& request, json::JsonRpcResponse*)
 {
    // read params
    json::Object generalPrefs, historyPrefs, editingPrefs, packagesPrefs,
-                projectsPrefs, sourceControlPrefs, compilePdfPrefs;
+                projectsPrefs, sourceControlPrefs, compilePdfPrefs,
+                terminalPrefs;
    Error error = json::readObjectParam(request.params, 0,
                               "general_prefs", &generalPrefs,
                               "history_prefs", &historyPrefs,
@@ -270,7 +267,8 @@ Error setPrefs(const json::JsonRpcRequest& request, json::JsonRpcResponse*)
                               "packages_prefs", &packagesPrefs,
                               "projects_prefs", &projectsPrefs,
                               "source_control_prefs", &sourceControlPrefs,
-                              "compile_pdf_prefs", &compilePdfPrefs);
+                              "compile_pdf_prefs", &compilePdfPrefs,
+                              "terminal_prefs", &terminalPrefs);
    if (error)
       return error;
    json::Object uiPrefs;
@@ -301,6 +299,15 @@ Error setPrefs(const json::JsonRpcRequest& request, json::JsonRpcResponse*)
    // detect if lastDotValue changed
    bool lastDotValueChanged = userSettings().showLastDotValue() != showLastDotValue;
 
+   // detect if initialWorkingDir changed
+   // we consider it changed if it has ever been written
+   // or if it has not been written and the value is different than the session default
+   bool initialWorkingDirChanged = false;
+   if (!userSettings().initialWorkingDirectory("").empty())
+      initialWorkingDirChanged = true;
+   else if (initialWorkingDir != session::options().defaultWorkingDir())
+      initialWorkingDirChanged = true;
+
    // update settings
    userSettings().beginUpdate();
    userSettings().setShowUserHomePage(showUserHomePage);
@@ -309,7 +316,8 @@ Error setPrefs(const json::JsonRpcRequest& request, json::JsonRpcResponse*)
    userSettings().setLoadRData(loadRData);
    userSettings().setRprofileOnResume(rProfileOnResume);
    userSettings().setShowLastDotValue(showLastDotValue);
-   userSettings().setInitialWorkingDirectory(FilePath(initialWorkingDir));
+   if (initialWorkingDirChanged)
+      userSettings().setInitialWorkingDirectory(FilePath(initialWorkingDir));
    userSettings().endUpdate();
 
    // refresh environment if lastDotValueChanged
@@ -411,14 +419,13 @@ Error setPrefs(const json::JsonRpcRequest& request, json::JsonRpcResponse*)
    userSettings().endUpdate();
 
    // read and set source control prefs
-   bool vcsEnabled, useGitBash;
+   bool vcsEnabled;
    std::string gitExe, svnExe, terminalPath;
    error = json::readObject(sourceControlPrefs,
                             "vcs_enabled", &vcsEnabled,
                             "git_exe_path", &gitExe,
                             "svn_exe_path", &svnExe,
-                            "terminal_path", &terminalPath,
-                            "use_git_bash", &useGitBash);
+                            "terminal_path", &terminalPath);
    if (error)
       return error;
    userSettings().beginUpdate();
@@ -442,8 +449,6 @@ Error setPrefs(const json::JsonRpcRequest& request, json::JsonRpcResponse*)
    else
       userSettings().setVcsTerminalPath(terminalFilePath);
 
-   userSettings().setVcsUseGitBash(useGitBash);
-
    userSettings().endUpdate();
 
 
@@ -459,6 +464,23 @@ Error setPrefs(const json::JsonRpcRequest& request, json::JsonRpcResponse*)
    userSettings().setEnableLaTeXShellEscape(enableShellEscape);
    userSettings().endUpdate();
 
+   // read and update terminal prefs
+   int defaultTerminalShell;
+   std::string customShellPath;
+   std::string customShellOptions;
+   error = json::readObject(terminalPrefs,
+                            "default_shell", &defaultTerminalShell,
+                            "shell_exe_path", &customShellPath,
+                            "shell_exe_options", &customShellOptions);
+   if (error)
+      return error;
+   userSettings().beginUpdate();
+   userSettings().setDefaultTerminalShellValue(
+      static_cast<console_process::TerminalShell::TerminalShellType>(defaultTerminalShell));
+   userSettings().setCustomShellCommand(customShellPath);
+   userSettings().setCustomShellOptions(customShellOptions);
+   userSettings().endUpdate();
+
    // set ui prefs
    userSettings().setUiPrefs(uiPrefs);
 
@@ -572,20 +594,23 @@ Error getRPrefs(const json::JsonRpcRequest& request,
       terminalPath = detectedTerminalPath();
    sourceControlPrefs["terminal_path"] = terminalPath.absolutePath();
 
-   sourceControlPrefs["use_git_bash"] = userSettings().vcsUseGitBash();
-
    FilePath sshKeyDir = modules::source_control::defaultSshKeyDir();
    FilePath rsaSshKeyPath = sshKeyDir.childPath("id_rsa");
    sourceControlPrefs["rsa_key_path"] =
                   module_context::createAliasedPath(rsaSshKeyPath);
    sourceControlPrefs["have_rsa_key"] = rsaSshKeyPath.exists();
 
-
    // get compile pdf prefs
    json::Object compilePdfPrefs;
    compilePdfPrefs["clean_output"] = userSettings().cleanTexi2DviOutput();
    compilePdfPrefs["enable_shell_escape"] = userSettings().enableLaTeXShellEscape();
 
+   // get terminal prefs
+   json::Object terminalPrefs;
+   terminalPrefs["default_shell"] = userSettings().defaultTerminalShellValue();
+   terminalPrefs["shell_exe_path"] = userSettings().customShellCommand().absolutePath();
+   terminalPrefs["shell_exe_options"] = userSettings().customShellOptions();
+
    // initialize and set result object
    json::Object result;
    result["general_prefs"] = generalPrefs;
@@ -597,30 +622,13 @@ Error getRPrefs(const json::JsonRpcRequest& request,
    result["compile_pdf_prefs"] = compilePdfPrefs;
    result["spelling_prefs_context"] =
                   session::modules::spelling::spellingPrefsContextAsJson();
+   result["terminal_prefs"] = terminalPrefs;
 
    pResponse->setResult(result);
 
    return Success();
 }
 
-template <typename T>
-void ammendShellPaths(T* pTarget)
-{
-   // non-path git bin dir
-   std::string gitBinDir = git::nonPathGitBinDir();
-   if (!gitBinDir.empty())
-      core::system::addToPath(pTarget, gitBinDir);
-
-   // non-path svn bin dir
-   std::string svnBinDir = svn::nonPathSvnBinDir();
-   if (!svnBinDir.empty())
-      core::system::addToPath(pTarget, svnBinDir);
-
-   // msys_ssh path
-   core::system::addToPath(pTarget,
-                           session::options().msysSshPath().absolutePath());
-}
-
 Error getTerminalOptions(const json::JsonRpcRequest& request,
                          json::JsonRpcResponse* pResponse)
 {
@@ -630,12 +638,23 @@ Error getTerminalOptions(const json::JsonRpcRequest& request,
 
 #if defined(_WIN32)
 
-   // if we are using git bash then return its path
-   if (git::isGitEnabled() && userSettings().vcsUseGitBash())
+   // Use default terminal setting for shell
+   console_process::AvailableTerminalShells shells;
+   console_process::TerminalShell shell;
+
+   if (shells.getInfo(userSettings().defaultTerminalShellValue(), &shell))
    {
-      FilePath gitExePath = git::detectedGitExePath();
-      if (!gitExePath.empty())
-         terminalPath = gitExePath.parent().childPath("sh.exe");
+      terminalPath = shell.path;
+   }
+
+   // last-ditch, use system shell
+   if (!terminalPath.exists())
+   {
+      console_process::TerminalShell sysShell;
+      if (console_process::AvailableTerminalShells::getSystemShell(&sysShell))
+      {
+         terminalPath = sysShell.path;
+      }
    }
 
 #elif defined(__APPLE__)
@@ -764,8 +783,7 @@ Error createSshKey(const json::JsonRpcRequest& request,
    core::system::environment(&childEnv);
 
    // set HOME to USERPROFILE
-   std::string userProfile = core::system::getenv(childEnv, "USERPROFILE");
-   core::system::setenv(&childEnv, "HOME", userProfile);
+   core::system::setHomeToUserProfile(&childEnv);
 
    // add msys_ssh to path
    core::system::addToPath(&childEnv,
@@ -851,66 +869,6 @@ void editFilePostback(const std::string& file,
    cont(EXIT_SUCCESS, "");
 }
 
-Error startShellDialog(const json::JsonRpcRequest& request,
-                       json::JsonRpcResponse* pResponse)
-{
-#ifndef _WIN32
-   using namespace session::module_context;
-   using namespace session::console_process;
-
-   // configure environment for shell
-   core::system::Options shellEnv;
-   core::system::environment(&shellEnv);
-
-   // set dumb terminal
-   core::system::setenv(&shellEnv, "TERM", "dumb");
-
-   // set prompt
-   std::string path = module_context::createAliasedPath(
-                                 module_context::safeCurrentPath());
-   std::string prompt = (path.length() > 30) ? "\\W$ " : "\\w$ ";
-   core::system::setenv(&shellEnv, "PS1", prompt);
-
-   // disable screen oriented facillites
-   core::system::unsetenv(&shellEnv, "EDITOR");
-   core::system::unsetenv(&shellEnv, "VISUAL");
-   core::system::setenv(&shellEnv, "PAGER", "/bin/cat");
-
-   core::system::setenv(&shellEnv, "GIT_EDITOR", s_editFileCommand);
-   core::system::setenv(&shellEnv, "SVN_EDITOR", s_editFileCommand);
-
-   // ammend shell paths as appropriate
-   ammendShellPaths(&shellEnv);
-
-   // set options
-   core::system::ProcessOptions options;
-   options.workingDir = module_context::shellWorkingDirectory();
-   options.environment = shellEnv;
-
-   // configure bash command
-   core::shell_utils::ShellCommand bashCommand("/bin/bash");
-   bashCommand << "--norc";
-
-   // run process
-   boost::shared_ptr<ConsoleProcess> ptrProc =
-               ConsoleProcess::create(bashCommand,
-                                      options,
-                                      "Shell",
-                                      true,
-                                      InteractionAlways,
-                                      console_process::kDefaultMaxOutputLines);
-
-   ptrProc->onExit().connect(boost::bind(
-                              &source_control::enqueueRefreshEvent));
-
-   pResponse->setResult(ptrProc->toJson());
-
-   return Success();
-#else // not supported on Win32
-   return Error(json::errc::InvalidRequest, ERROR_LOCATION);
-#endif
-}
-
 Error setCRANMirror(const json::JsonRpcRequest& request,
                     json::JsonRpcResponse* pResponse)
 {
@@ -1030,14 +988,12 @@ Error initialize()
       (bind(registerRpcMethod, "set_cran_mirror", setCRANMirror))
       (bind(registerRpcMethod, "get_terminal_options", getTerminalOptions))
       (bind(registerRpcMethod, "create_ssh_key", createSshKey))
-      (bind(registerRpcMethod, "start_shell_dialog", startShellDialog))
       (bind(registerRpcMethod, "execute_code", executeCode));
    return initBlock.execute();
 }
 
 
-} // namepsace workbench
+} // namespace workbench
 } // namespace modules
-} // namesapce session
+} // namespace session
 } // namespace rstudio
-
