@@ -1,7 +1,7 @@
 /*
  * DataViewer.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2009-17 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -20,6 +20,7 @@
 #include <sstream>
 
 #include <boost/bind.hpp>
+#include <boost/foreach.hpp>
 #include <boost/format.hpp>
 #include <boost/algorithm/string/predicate.hpp>
 
@@ -56,9 +57,6 @@
 // separates filter type from contents (e.g. "numeric|12-25")
 #define kFilterSeparator "|"
 
-// the largest number of columns we're willing to display
-#define MAX_COLS 100  
-
 // the largest number of factor values we're willing to display (after this
 // point the column's text is searched as though it were a character column)
 #define MAX_FACTORS 64
@@ -165,8 +163,8 @@ bool isFilterSubset(const std::string& outer, const std::string& inner)
       // check the components for range inclusion
       boost::regex numFilter("(-?\\d+\\.?\\d*)_(-?\\d+\\.?\\d*)");
       boost::smatch innerMatch, outerMatch;
-      if (boost::regex_search(innerValue, innerMatch, numFilter) &&
-          boost::regex_search(outerValue, outerMatch, numFilter))
+      if (regex_utils::search(innerValue, innerMatch, numFilter) &&
+          regex_utils::search(outerValue, outerMatch, numFilter))
       {
          // for numeric filters, the inner is a subset if its lower bound (1)
          // is larger than the outer lower bound, and the upper bound (2) is
@@ -299,7 +297,7 @@ std::map<std::string, CachedFrame> s_cachedFrames;
 
 std::string viewerCacheDir() 
 {
-   return module_context::scopedScratchPath().childPath(kViewerCacheDir)
+   return module_context::sessionScratchPath().childPath(kViewerCacheDir)
       .absolutePath();
 }
 
@@ -385,8 +383,10 @@ SEXP rs_viewData(SEXP dataSEXP, SEXP captionSEXP, SEXP nameSEXP, SEXP envSEXP,
       
       // attempt to cast to a data frame
       SEXP dataFrameSEXP = R_NilValue;
-      Error error = r::exec::RFunction("as.data.frame", dataSEXP).call(
-            &dataFrameSEXP, &protect);
+      r::exec::RFunction asDataFrame("as.data.frame");
+      asDataFrame.addParam("x", dataSEXP);
+      asDataFrame.addParam("optional", true);  // don't require column names
+      Error error = asDataFrame.call(&dataFrameSEXP, &protect);
       if (error) 
       {
          // caught below
@@ -439,8 +439,7 @@ json::Value getCols(SEXP dataSEXP)
    SEXP colsSEXP = R_NilValue;
    r::sexp::Protect protect;
    json::Value result;
-   Error error = r::exec::RFunction(".rs.describeCols", dataSEXP, MAX_COLS, 
-         MAX_FACTORS)
+   Error error = r::exec::RFunction(".rs.describeCols", dataSEXP, MAX_FACTORS)
       .call(&colsSEXP, &protect);
    if (error || colsSEXP == R_NilValue) 
    {
@@ -491,7 +490,6 @@ json::Value getData(SEXP dataSEXP, const http::Fields& fields)
    int nrow = safeDim(dataSEXP, DIM_ROWS);
    int ncol = safeDim(dataSEXP, DIM_COLS);
    int filteredNRow = 0;
-   ncol = std::min(ncol, MAX_COLS);
 
    // extract filters
    std::vector<std::string> filters;
@@ -915,10 +913,6 @@ void onClientInit()
       // not fatal, log and continue
       LOG_ERROR(error);
    }
-   else
-   {
-      module_context::events().onPermissionsChanged(cacheDir);
-   }
 #endif
 }
 
@@ -943,6 +937,62 @@ void onDocPendingRemove(
       LOG_ERROR(error);
 }
 
+void onDeferredInit(bool newSession)
+{
+   // get all the cache keys in the source database
+   std::vector<boost::shared_ptr<source_database::SourceDocument> > docs;
+   Error error = source_database::list(&docs);
+   if (error)
+   {
+      LOG_ERROR(error);
+      return;
+   }
+
+   std::vector<std::string> sourceKeys;
+   BOOST_FOREACH(boost::shared_ptr<source_database::SourceDocument> pDoc, docs)
+   {
+      std::string key = pDoc->getProperty("cacheKey");
+      if (!key.empty())
+         sourceKeys.push_back(key);
+   }
+   
+   // get all the cache keys in the cache
+   FilePath cache(viewerCacheDir());
+   std::vector<FilePath> cacheFiles;
+   if (cache.exists())
+   {
+      Error error = cache.children(&cacheFiles);
+      if (error)
+      {
+         LOG_ERROR(error);
+         return;
+      }
+   }
+
+   std::vector<std::string> cacheKeys;
+   BOOST_FOREACH(const FilePath& cacheFile, cacheFiles)
+   {
+      cacheKeys.push_back(cacheFile.stem());
+   }
+
+   // sort each set of keys (so we can diff the sets below)
+   std::sort(sourceKeys.begin(), sourceKeys.end());
+   std::sort(cacheKeys.begin(), cacheKeys.end());
+
+   std::vector<std::string> orphanKeys;
+   std::set_difference(cacheKeys.begin(), cacheKeys.end(),
+                       sourceKeys.begin(), sourceKeys.end(),
+                       std::back_inserter(orphanKeys));
+
+   // remove each key no longer bound to a source file
+   BOOST_FOREACH(const std::string& orphanKey, orphanKeys)
+   {
+      error = cache.complete(orphanKey + ".Rdata").removeIfExists();
+      if (error)
+         LOG_ERROR(error);
+   }
+}
+
 } // anonymous namespace
    
 Error initialize()
@@ -961,6 +1011,7 @@ Error initialize()
    module_context::events().onShutdown.connect(onShutdown);
    module_context::events().onDetectChanges.connect(onDetectChanges);
    module_context::events().onClientInit.connect(onClientInit);
+   module_context::events().onDeferredInit.connect(onDeferredInit);
    addSuspendHandler(SuspendHandler(onSuspend, onResume));
 
    using boost::bind;
@@ -991,6 +1042,6 @@ Error initialize()
 } // namespace viewer
 } // namespace data
 } // namespace modules
-} // namesapce session
+} // namespace session
 } // namespace rstudio
 
