@@ -33,7 +33,7 @@ core::Error initialize();
 } // namespace viewer
 } // namespace data
 } // namespace modules
-} // namesapce session
+} // namespace session
 } // namespace rstudio
 
 #endif // SESSION_DATA_VIEWER_HPP
