@@ -164,7 +164,13 @@ public:
                return Success();
          }
          else
-            return executeCurrentUnit(mode);
+         {
+            // if unit is executing code, execute more code
+            if (!execUnit_->executingCode().empty())
+               return executeCurrentUnit(mode);
+            else
+               return Success();
+         }
       }
 
       return executeNextUnit(mode);
@@ -429,10 +435,11 @@ private:
          else
          {
             execUnit_ = unit;
+
             enqueueExecStateChanged(ChunkExecStarted, options.chunkOptions());
             error = executeAlternateEngineChunk(
-               unit->docId(), unit->chunkId(), ctx, engine, innerCode, 
-               options.mergedOptions());
+               unit->docId(), unit->chunkId(), ctx, docQueue->workingDir(),
+               engine, innerCode, options.mergedOptions());
             if (error)
                LOG_ERROR(error);
          }
@@ -592,7 +599,7 @@ private:
 
          // update running queue if present
          if (!queue_.empty() && !workingDir.empty())
-            queue_.front()->setWorkingDir(workingDir);
+            queue_.front()->setWorkingDir(workingDir, SetupChunkDir);
       }
       else if (!error)
       {
@@ -602,8 +609,10 @@ private:
                kChunkWorkingDir, json::Value());
          if (error)
             LOG_ERROR(error);
-         if (!queue_.empty())
-            queue_.front()->setWorkingDir("");
+
+         // if the running queue got its working directory from the setup chunk, clear it
+         if (!queue_.empty() && queue_.front()->getWorkingDirSource() == SetupChunkDir)
+            queue_.front()->setWorkingDir("", SetupChunkDir);
       }
 
       error = r::exec::RFunction(".rs.defaultChunkOptions")
