@@ -22,20 +22,22 @@ import com.google.gwt.resources.client.ImageResource;
 import com.google.gwt.safehtml.shared.SafeHtml;
 import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
 import com.google.gwt.user.client.Command;
-import com.google.gwt.user.client.ui.AbstractImagePrototype;
 import com.google.gwt.user.client.ui.Image;
 import com.google.gwt.user.client.ui.MenuItem;
 
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.SafeHtmlUtil;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.command.KeyboardShortcut.KeySequence;
 import org.rstudio.core.client.dom.DomUtils;
+import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.theme.res.ThemeResources;
 import org.rstudio.core.client.theme.res.ThemeStyles;
 import org.rstudio.core.client.widget.Toolbar;
 import org.rstudio.core.client.widget.ToolbarButton;
 import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.application.Desktop;
+import org.rstudio.studio.client.application.ui.RStudioThemes;
 import org.rstudio.studio.client.common.satellite.Satellite;
 import org.rstudio.studio.client.common.satellite.SatelliteManager;
 
@@ -368,9 +370,16 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
    {
       if (isCheckable())
       {
-         return isChecked() ? 
-            ThemeResources.INSTANCE.menuCheck() :
-            null;
+         if (RStudioThemes.isFlat() && RStudioThemes.isEditorDark()) {
+            return isChecked() ? 
+               new ImageResource2x(ThemeResources.INSTANCE.menuCheckInverted2x()) :
+               null;
+         }
+         else {
+            return isChecked() ? 
+               new ImageResource2x(ThemeResources.INSTANCE.menuCheck2x()) :
+               null;
+         }
       } 
       else
       {
@@ -458,6 +467,21 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
       return formatMenuLabel(icon, label, false, shortcut);
    }
    
+   public static String formatMenuLabelWithStyle(ImageResource icon, 
+                                                 String label,
+                                                 String shortcut,
+                                                 String styleName)
+   {
+      return formatMenuLabel(icon, 
+            label, 
+            false,
+            shortcut, 
+            null, 
+            null,
+            null,
+            styleName);
+   }
+   
    public static String formatMenuLabel(ImageResource icon, 
          String label,
          boolean html,
@@ -488,7 +512,8 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
                              shortcut, 
                              null, 
                              rightImage,
-                             rightImageDesc);
+                             rightImageDesc,
+                             null);
    }
    
    public static String formatMenuLabel(ImageResource icon, 
@@ -505,7 +530,7 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
                                         String shortcut, 
                                         Integer iconOffsetY)
    {
-      return formatMenuLabel(icon, label, html, shortcut, iconOffsetY, null, null);
+      return formatMenuLabel(icon, label, html, shortcut, iconOffsetY, null, null, null);
    }
    
    public static String formatMenuLabel(ImageResource icon, 
@@ -514,19 +539,33 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
                                          String shortcut, 
                                          Integer iconOffsetY,
                                          ImageResource rightImage,
-                                         String rightImageDesc)
+                                         String rightImageDesc,
+                                         String styleName)
    {
       StringBuilder text = new StringBuilder();
-      int topOffset = -10;
+      int topOffset = -2;
       if (iconOffsetY != null)
          topOffset += iconOffsetY;
-      text.append("<table border=0 cellpadding=0 cellspacing=0 width='100%'><tr>");
+      text.append("<table ");
+      if (label != null)
+      {
+         text.append("id=\"" + ElementIds.idFromLabel(label) + "_command\" ");
+      }
+      
+      if (styleName != null)
+      {
+         text.append("class='" + styleName + "' ");
+      }
+      
+      text.append("border=0 cellpadding=0 cellspacing=0 width='100%'><tr>");
 
-      text.append("<td width=\"25\"><div style=\"width: 25px; margin-top: " +
+      text.append("<td width=\"25\" style=\"vertical-align: top\">" +
+                  "<div style=\"width: 25px; margin-top: " +
                   topOffset + "px; margin-bottom: -10px\">");
       if (icon != null)
       {
-         text.append(AbstractImagePrototype.create(icon).getHTML());
+         SafeHtml imageHtml = createMenuImageHtml(icon);
+         text.append(imageHtml.asString());
       }
       else
       {
@@ -597,6 +636,19 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
       sb.append(SafeHtmlUtil.createOpenTag("img",
         "class", ThemeStyles.INSTANCE.menuRightImage(),
         "title", StringUtil.notNull(desc),
+        "width", Integer.toString(image.getWidth()),
+        "height", Integer.toString(image.getHeight()),
+        "src", image.getSafeUri().asString()));
+      sb.appendHtmlConstant("</img>");   
+      return sb.toSafeHtml();
+   }
+
+   private static SafeHtml createMenuImageHtml(ImageResource image)
+   {
+      SafeHtmlBuilder sb = new SafeHtmlBuilder();
+      sb.append(SafeHtmlUtil.createOpenTag("img",
+        "width", Integer.toString(image.getWidth()),
+        "height", Integer.toString(image.getHeight()),
         "src", image.getSafeUri().asString()));
       sb.appendHtmlConstant("</img>");   
       return sb.toSafeHtml();
