@@ -1,7 +1,7 @@
 /*
  * WebApplicationHeader.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2009-17 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -37,6 +37,7 @@ import com.google.inject.Provider;
 import org.rstudio.core.client.BrowseCap;
 import org.rstudio.core.client.command.*;
 import org.rstudio.core.client.command.impl.WebMenuCallback;
+import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.theme.res.ThemeResources;
 import org.rstudio.core.client.theme.res.ThemeStyles;
 import org.rstudio.core.client.widget.HyperlinkLabel;
@@ -77,7 +78,7 @@ public class WebApplicationHeader extends Composite
                   final Commands commands,
                   EventBus eventBus,
                   GlobalDisplay globalDisplay,
-                  ThemeResources themeResources,
+                  final ThemeResources themeResources,
                   final Session session,
                   Provider<CodeSearch> pCodeSearch)
    {
@@ -94,12 +95,12 @@ public class WebApplicationHeader extends Composite
       outerPanel_.getElement().getStyle().setPosition(Position.RELATIVE);
       
       // large logo
-      logoLarge_ = new Image(ThemeResources.INSTANCE.rstudio());
+      logoLarge_ = new Image(new ImageResource2x(ThemeResources.INSTANCE.rstudio2x()));
       ((ImageElement)logoLarge_.getElement().cast()).setAlt("RStudio");
       logoLarge_.getElement().getStyle().setBorderWidth(0, Unit.PX);
       
       // small logo
-      logoSmall_ = new Image(ThemeResources.INSTANCE.rstudio_small());
+      logoSmall_ = new Image(new ImageResource2x(ThemeResources.INSTANCE.rstudio_small2x()));
       ((ImageElement)logoSmall_.getElement().cast()).setAlt("RStudio");
       logoSmall_.getElement().getStyle().setBorderWidth(0, Unit.PX);
 
@@ -112,6 +113,8 @@ public class WebApplicationHeader extends Composite
       style.setTextDecoration(TextDecoration.NONE);
       style.setOutlineWidth(0, Unit.PX);
 
+      logoAnchor_.setStylePrimaryName(themeResources.themeStyles().logoAnchor());
+
       // header container
       headerBarPanel_ = new HorizontalPanel() ;
       headerBarPanel_.setStylePrimaryName(themeResources.themeStyles().header());
@@ -171,6 +174,14 @@ public class WebApplicationHeader extends Composite
          {
             SessionInfo sessionInfo = session.getSessionInfo();
             
+            hostedMode_ = !sessionInfo.getAllowFullUI();
+            
+            if (hostedMode_)
+            {
+               mainMenu_.addStyleName(themeResources.themeStyles().noLogo());
+               toolbar_.addStyleName(themeResources.themeStyles().noLogo());
+            }
+            
             // complete toolbar initialization
             toolbar_.completeInitialization(sessionInfo);
              
@@ -191,8 +202,8 @@ public class WebApplicationHeader extends Composite
                logoAnchor_.setHref(logoTargetUrl_);
                logoAnchor_.setTitle("RStudio Server Home");
 
-               logoLarge_.setResource(ThemeResources.INSTANCE.rstudio_home());
-               logoSmall_.setResource(ThemeResources.INSTANCE.rstudio_home_small());
+               logoLarge_.setResource(new ImageResource2x(ThemeResources.INSTANCE.rstudio_home2x()));
+               logoSmall_.setResource(new ImageResource2x(ThemeResources.INSTANCE.rstudio_home_small2x()));
             }
             else
             {
@@ -215,6 +226,7 @@ public class WebApplicationHeader extends Composite
                                    eventBus,
                                    pCodeSearch);
       toolbar_.addStyleName(themeResources.themeStyles().webGlobalToolbar());
+      toolbar_.getWrapper().addStyleName(themeResources.themeStyles().webGlobalToolbarWrapper());
       
       // create host for project commands
       projectBarCommandsPanel_ = new HorizontalPanel();
@@ -226,27 +238,32 @@ public class WebApplicationHeader extends Composite
     
    public void showToolbar(boolean showToolbar)
    {
+      toolbarVisible_ = showToolbar;
       outerPanel_.clear();
       
       if (showToolbar)
       {
-         logoAnchor_.getElement().removeAllChildren();
-         logoAnchor_.getElement().appendChild(logoLarge_.getElement());
-         outerPanel_.add(logoAnchor_);
+         if (!hostedMode_)
+         {
+            logoAnchor_.getElement().removeAllChildren();
+            logoAnchor_.getElement().appendChild(logoLarge_.getElement());
+            outerPanel_.add(logoAnchor_);
+         }
          HeaderPanel headerPanel = new HeaderPanel(headerBarPanel_, toolbar_);
          outerPanel_.add(headerPanel);
-         mainMenu_.getElement().getStyle().setMarginLeft(18, Unit.PX);
          preferredHeight_ = 65;
          showProjectMenu(false);
       }
       else
       {
-         logoAnchor_.getElement().removeAllChildren();
-         logoAnchor_.getElement().appendChild(logoSmall_.getElement());
-         outerPanel_.add(logoAnchor_);
+         if (!hostedMode_)
+         {
+            logoAnchor_.getElement().removeAllChildren();
+            logoAnchor_.getElement().appendChild(logoSmall_.getElement());
+            outerPanel_.add(logoAnchor_);
+         }
          MenubarPanel menubarPanel = new MenubarPanel(headerBarPanel_);
          outerPanel_.add(menubarPanel);
-         mainMenu_.getElement().getStyle().setMarginLeft(0, Unit.PX);
          preferredHeight_ = 45;
          showProjectMenu(true);
       }
@@ -256,7 +273,7 @@ public class WebApplicationHeader extends Composite
    
    public boolean isToolbarVisible()
    {
-      return !projectMenuButton_.isVisible();
+      return toolbarVisible_;
    }
    
    public void focusGoToFunction()
@@ -266,6 +283,10 @@ public class WebApplicationHeader extends Composite
    
    private void showProjectMenu(boolean show)
    {
+      if (hostedMode_)
+      {
+         show = false;
+      }
       projectMenuSeparator_.setVisible(show);
       projectMenuButton_.setVisible(show);
    }
@@ -381,7 +402,7 @@ public class WebApplicationHeader extends Composite
    private void initCommandsPanel(final SessionInfo sessionInfo)
    {  
       // add username 
-      if (sessionInfo.getShowIdentity())
+      if (sessionInfo.getShowIdentity() && sessionInfo.getAllowFullUI())
       {
          ToolbarLabel usernameLabel = new ToolbarLabel();
          usernameLabel.getElement().getStyle().setMarginRight(2, Unit.PX);
@@ -393,7 +414,7 @@ public class WebApplicationHeader extends Composite
          usernameLabel.setText(userIdentity);
          headerBarCommandsPanel_.add(usernameLabel);
         
-         ToolbarButton signOutButton = new ToolbarButton(RESOURCES.signOut(),
+         ToolbarButton signOutButton = new ToolbarButton(new ImageResource2x(RESOURCES.signOut2x()),
               new ClickHandler() {
             public void onClick(ClickEvent event)
             {
@@ -408,7 +429,8 @@ public class WebApplicationHeader extends Composite
       
       overlay_.addCommands(this);
       
-      headerBarCommandsPanel_.add(commands_.quitSession().createToolbarButton());
+      if (sessionInfo.getAllowFullUI())
+         headerBarCommandsPanel_.add(commands_.quitSession().createToolbarButton());
    }
 
    private Widget createCommandSeparator()
@@ -510,7 +532,8 @@ public class WebApplicationHeader extends Composite
    
    interface Resources extends ClientBundle
    {
-      ImageResource signOut();
+      @Source("signOut_2x.png")
+      ImageResource signOut2x();
    }
    
    private static final Resources RESOURCES =  
@@ -534,4 +557,6 @@ public class WebApplicationHeader extends Composite
    private GlobalDisplay globalDisplay_;
    private Commands commands_; 
    private WebApplicationHeaderOverlay overlay_;
+   private boolean hostedMode_;
+   private boolean toolbarVisible_;
 }
