@@ -54,7 +54,7 @@ public class DependencyManager implements InstallShinyEvent.Handler,
       DependencyRequest(
             String progressCaptionIn,
             String userActionIn,
-            CommandWith2Args<String,Command> userPromptIn,
+            CommandWith2Args<String,CommandWithArg<Boolean>> userPromptIn,
             Dependency[] dependenciesIn,
             boolean silentEmbeddedUpdateIn,
             CommandWithArg<Boolean> onCompleteIn)
@@ -68,7 +68,7 @@ public class DependencyManager implements InstallShinyEvent.Handler,
       }
       String progressCaption;
       String userAction;
-      CommandWith2Args<String,Command> userPrompt;
+      CommandWith2Args<String,CommandWithArg<Boolean>> userPrompt;
       Dependency[] dependencies;
       boolean silentEmbeddedUpdate;
       CommandWithArg<Boolean> onComplete;
@@ -89,10 +89,10 @@ public class DependencyManager implements InstallShinyEvent.Handler,
    }
    
    public void withDependencies(String progressCaption,
-                                CommandWith2Args<String,Command> userPrompt,
-                                Dependency[] dependencies, 
-                                boolean silentEmbeddedUpdate,
-                                CommandWithArg<Boolean> onComplete)
+        CommandWith2Args<String,CommandWithArg<Boolean>> userPrompt,
+        Dependency[] dependencies, 
+        boolean silentEmbeddedUpdate,
+        CommandWithArg<Boolean> onComplete)
    {
       withDependencies(progressCaption,
                        null,
@@ -138,7 +138,7 @@ public class DependencyManager implements InstallShinyEvent.Handler,
    
    public void withRSConnect(String userAction, 
          boolean requiresRmarkdown,
-         CommandWith2Args<String, Command> userPrompt, 
+         CommandWith2Args<String, CommandWithArg<Boolean>> userPrompt, 
          final CommandWithArg<Boolean> onCompleted)
    {
       // build dependency array
@@ -152,14 +152,14 @@ public class DependencyManager implements InstallShinyEvent.Handler,
       if (requiresRmarkdown)
          deps.addAll(rmarkdownDependencies());
       deps.add(Dependency.cranPackage("packrat", "0.4.8-1", true));
-      deps.add(Dependency.cranPackage("rsconnect", "0.6", true));
+      deps.add(Dependency.cranPackage("rsconnect", "0.8.5"));
       
       withDependencies(
         "Publishing",
         userAction,
         userPrompt,
         deps.toArray(new Dependency[deps.size()]),
-        true, // we want the embedded rsconnect package to be updated if needed
+        true, // silently update any embedded packages needed (none at present)
         onCompleted
       );
    }
@@ -205,7 +205,6 @@ public class DependencyManager implements InstallShinyEvent.Handler,
       ArrayList<Dependency> deps = new ArrayList<Dependency>();
       deps.add(Dependency.cranPackage("evaluate", "0.8"));
       deps.add(Dependency.cranPackage("digest", "0.6"));
-      deps.add(Dependency.cranPackage("formatR", "1.1"));
       deps.add(Dependency.cranPackage("highr", "0.3"));
       deps.add(Dependency.cranPackage("markdown", "0.7"));
       deps.add(Dependency.cranPackage("stringr", "0.6"));
@@ -214,11 +213,11 @@ public class DependencyManager implements InstallShinyEvent.Handler,
       deps.add(Dependency.cranPackage("htmltools", "0.3.5"));
       deps.add(Dependency.cranPackage("caTools", "1.14"));
       deps.add(Dependency.cranPackage("bitops", "1.0-6"));
-      deps.add(Dependency.cranPackage("knitr", "1.14", true));
+      deps.add(Dependency.cranPackage("knitr", "1.14"));
       deps.add(Dependency.cranPackage("jsonlite", "0.9.19"));
       deps.add(Dependency.cranPackage("base64enc", "0.1-3"));
       deps.add(Dependency.cranPackage("rprojroot", "1.0"));
-      deps.add(Dependency.cranPackage("rmarkdown", "1.2", true));
+      deps.add(Dependency.cranPackage("rmarkdown", "1.6"));
       return deps;
    }
    
@@ -231,22 +230,32 @@ public class DependencyManager implements InstallShinyEvent.Handler,
    public void withShiny(final String userAction, final Command command)
    {
       // create user prompt command
-      CommandWith2Args<String, Command> userPrompt =
-            new CommandWith2Args<String, Command>() {
+      CommandWith2Args<String, CommandWithArg<Boolean>> userPrompt =
+            new CommandWith2Args<String, CommandWithArg<Boolean>>() {
          @Override
-         public void execute(final String unmetDeps, final Command yesCommand)
+         public void execute(final String unmetDeps, 
+                             final CommandWithArg<Boolean> responseCommand)
          {
             globalDisplay_.showYesNoMessage(
               MessageDialog.QUESTION,
               "Install Shiny Package", 
               userAction + " requires installation of an updated version " +
               "of the shiny package.\n\nDo you want to install shiny now?",
-                  new Operation() {
-
+                  false, // include cancel
+                  new Operation() 
+                  {
                      @Override
                      public void execute()
                      {
-                        yesCommand.execute();
+                        responseCommand.execute(true);
+                     }
+                  },
+                  new Operation() 
+                  {
+                     @Override
+                     public void execute()
+                     {
+                        responseCommand.execute(false);
                      }
                   },
                   true);
@@ -313,7 +322,7 @@ public class DependencyManager implements InstallShinyEvent.Handler,
       deps.add(Dependency.cranPackage("R6", "2.0"));
       deps.add(Dependency.cranPackage("sourcetools", "0.1.5"));
       deps.add(Dependency.cranPackage("htmltools", "0.3.5"));
-      deps.add(Dependency.cranPackage("shiny", "0.13", true));
+      deps.add(Dependency.cranPackage("shiny", "1.0", true));
       return deps;
          }
    
@@ -621,13 +630,44 @@ public class DependencyManager implements InstallShinyEvent.Handler,
         }
      );
    }
+
+   public void withConnectionPackage(String connectionName,
+                                     String packageName,
+                                     String packageVersion,
+                                     final Operation operation)
+   {
+     withDependencies(
+        "Preparing Connection",
+        connectionName, 
+        connectionPackageDependenciesArray(packageName, packageVersion), 
+        false,
+        new CommandWithArg<Boolean>()
+        {
+           @Override
+           public void execute(Boolean succeeded)
+           {
+              if (succeeded)
+                 operation.execute();
+           }
+        }
+     );
+   }
+   
+   private Dependency[] connectionPackageDependenciesArray(String packageName,
+                                                           String packageVersion)
+   {
+    ArrayList<Dependency> deps = new ArrayList<Dependency>();
+      deps.add(Dependency.cranPackage(packageName, packageVersion));
+
+      return deps.toArray(new Dependency[deps.size()]);
+   }
    
    private void withDependencies(String progressCaption,
-                                 final String userAction,
-                                 final CommandWith2Args<String,Command> userPrompt,
-                                 Dependency[] dependencies, 
-                                 final boolean silentEmbeddedUpdate,
-                                 final CommandWithArg<Boolean> onComplete)
+         final String userAction,
+         final CommandWith2Args<String,CommandWithArg<Boolean>> userPrompt,
+         Dependency[] dependencies, 
+         final boolean silentEmbeddedUpdate,
+         final CommandWithArg<Boolean> onComplete)
    {
       // add the request to the queue rather than processing it right away; 
       // this frees us of the burden of trying to de-dupe requests for the
@@ -777,12 +817,12 @@ public class DependencyManager implements InstallShinyEvent.Handler,
                if (req.userPrompt != null)
                {
                   req.userPrompt.execute(describeDepPkgs(unsatisfiedDeps), 
-                         new Command()
+                         new CommandWithArg<Boolean>()
                          {
                            @Override
-                           public void execute()
+                           public void execute(Boolean arg)
                            {
-                              installCommand.execute(true);
+                              installCommand.execute(arg);
                            }
                          });
                }
