@@ -1,7 +1,7 @@
 /*
  * FileTypeRegistry.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2009-17 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,6 +18,7 @@ import java.util.HashMap;
 
 import org.rstudio.core.client.FilePosition;
 import org.rstudio.core.client.files.FileSystemItem;
+import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.studio.client.application.Desktop;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.common.GlobalDisplay;
@@ -43,17 +44,17 @@ public class FileTypeRegistry
 
    public static final TextFileType TEXT =
          new TextFileType("text", "Text File", EditorLanguage.LANG_PLAIN, "",
-                          ICONS.iconText(),
+                          new ImageResource2x(ICONS.iconText2x()),
                           true,
                           false, false, false, false, false, false, false, false, false, true, false, false);
 
    public static final TextFileType R =
          new RFileType("r_source", "R Script", EditorLanguage.LANG_R, ".R",
-                       ICONS.iconRdoc());
+                       new ImageResource2x(ICONS.iconRdoc2x()));
 
    public static final TextFileType RD =
       new TextFileType("r_doc", "Rd File", EditorLanguage.LANG_RDOC, ".Rd",
-                       ICONS.iconRd(),
+                       new ImageResource2x(ICONS.iconRd2x()),
                        true, // word-wrap
                        true, // source on save aka preview on save
                        false, false, false,
@@ -65,7 +66,7 @@ public class FileTypeRegistry
 
    public static final TextFileType DCF =
          new TextFileType("dcf", "DCF", EditorLanguage.LANG_DCF, ".dcf",
-                          ICONS.iconDCF(), false, false, false, false, false,
+                          new ImageResource2x(ICONS.iconDCF2x()), false, false, false, false, false,
                           false, false, false, false, false, false, false, false);
    
    public static final TextFileType STAN = new StanFileType();
@@ -77,195 +78,202 @@ public class FileTypeRegistry
 
    public static final TextFileType NAMESPACE =
      new TextFileType("r_namespace", "NAMESPACE", EditorLanguage.LANG_R, "",
-                      ICONS.iconText(), false, false, false, false, false,
+                      new ImageResource2x(ICONS.iconText2x()), false, false, false, false, false,
                       false, false, false, false, false, false, false, false);
 
    public static final TextFileType SWEAVE =
       new SweaveFileType("sweave", "R Sweave",
-          EditorLanguage.LANG_SWEAVE, ".Rnw",ICONS.iconRsweave());
+          EditorLanguage.LANG_SWEAVE, ".Rnw",new ImageResource2x(ICONS.iconRsweave2x()));
 
    public static final TexFileType TEX =
          new TexFileType("tex", "TeX", EditorLanguage.LANG_TEX, ".tex",
-                          ICONS.iconTex());
+                          new ImageResource2x(ICONS.iconTex2x()));
 
    public static final PlainTextFileType RHISTORY =
       new PlainTextFileType("r_history", "R History", ".Rhistory",
-                            ICONS.iconRhistory(),
+                            new ImageResource2x(ICONS.iconRhistory2x()),
                             true);
 
    public static final RWebContentFileType RMARKDOWN =
          new RWebContentFileType("r_markdown", "R Markdown", EditorLanguage.LANG_RMARKDOWN,
-                              ".Rmd", ICONS.iconRmarkdown(), true);
+                              ".Rmd", new ImageResource2x(ICONS.iconRmarkdown2x()), true);
    
    public static final RWebContentFileType RNOTEBOOK =
          new RWebContentFileType("r_notebook", "R Notebook", EditorLanguage.LANG_RMARKDOWN,
-                                 ".nb.html", ICONS.iconRnotebook(), true);
+                                 ".nb.html", new ImageResource2x(ICONS.iconRnotebook2x()), true);
 
    public static final RWebContentFileType RPRESENTATION = new RPresentationFileType();
 
    public static final WebContentFileType MARKDOWN =
       new WebContentFileType("markdown", "Markdown", EditorLanguage.LANG_MARKDOWN,
-                           ".md", ICONS.iconMarkdown(), true);
+                           ".md", new ImageResource2x(ICONS.iconMarkdown2x()), true);
 
 
    public static final RWebContentFileType RHTML =
          new RWebContentFileType("r_html", "R HTML", EditorLanguage.LANG_RHTML,
-                              ".Rhtml", ICONS.iconRhtml(), false);
+                              ".Rhtml", new ImageResource2x(ICONS.iconRhtml2x()), false);
 
    public static final WebContentFileType HTML =
          new WebContentFileType("html", "HTML", EditorLanguage.LANG_HTML,
-                              ".html", ICONS.iconHTML(), false);
+                              ".html", new ImageResource2x(ICONS.iconHTML2x()), false);
 
    public static final TextFileType CSS =
          new TextFileType("css", "CSS", EditorLanguage.LANG_CSS, ".css",
-                          ICONS.iconCss(),
+                          new ImageResource2x(ICONS.iconCss2x()),
                           true,
                           false, false, false, false, false, false, false, false, false, false, false, false);
 
    public static final TextFileType JS =
          new TextFileType("js", "JavaScript", EditorLanguage.LANG_JAVASCRIPT, ".js",
-                          ICONS.iconJavascript(),
+                          new ImageResource2x(ICONS.iconJavascript2x()),
                           true,
                           false, false, false, false, false, false, false, false, false, false, false, false);
    
    public static final TextFileType JSON =
          new TextFileType("json", "JSON", EditorLanguage.LANG_JAVASCRIPT, ".json",
-                          ICONS.iconJavascript(),
+                          new ImageResource2x(ICONS.iconJavascript2x()),
                           true,
                           false, false, false, false, false, false, false, false, false, false, false, false);
    
 
    public static final TextFileType PYTHON = new ScriptFileType(
-     "python", "Python", EditorLanguage.LANG_PYTHON, ".py",ICONS.iconPython(),
-     "python", true);
+     "python", "Python", EditorLanguage.LANG_PYTHON, ".py",new ImageResource2x(ICONS.iconPython2x()),
+     "python", false, true);
 
    public static final TextFileType SQL =
          new TextFileType("sql", "SQL", EditorLanguage.LANG_SQL, ".sql",
-                          ICONS.iconSql(), false, false, false, false, false,
+                          new ImageResource2x(ICONS.iconSql2x()), false, false, false, false, false,
                           false, false, false, false, false, false, false, false);
 
    public static final TextFileType SH = new ScriptFileType(
-         "sh", "Shell", EditorLanguage.LANG_SH, ".sh", ICONS.iconSh(),
-         null, false);
+         "sh", "Shell", EditorLanguage.LANG_SH, ".sh", new ImageResource2x(ICONS.iconSh2x()),
+         null, true, false);
+   
+   public static final TextFileType TOML =
+         new TextFileType("toml", "TOML", EditorLanguage.LANG_TOML, ".toml",
+                          new ImageResource2x(ICONS.iconToml2x()), false, false, false, false, false,
+                          false, false, false, false, false, false, false, false);
    
    public static final TextFileType YAML =
          new TextFileType("yaml", "YAML", EditorLanguage.LANG_YAML, ".yml",
-                          ICONS.iconYaml(), false, false, false, false, false,
+                          new ImageResource2x(ICONS.iconYaml2x()), false, false, false, false, false,
                           false, false, false, false, false, false, false, false);
-
+   
    public static final TextFileType XML =
          new TextFileType("xml", "XML", EditorLanguage.LANG_XML, ".xml",
-                          ICONS.iconXml(), false, false, false, false, false,
+                          new ImageResource2x(ICONS.iconXml2x()), false, false, false, false, false,
                           false, false, false, false, false, false, false, false);
    
-   public static final TextFileType H = new CppFileType("h", ".h", ICONS.iconH(), true, false);
-   public static final TextFileType C = new CppFileType("c", ".c", ICONS.iconC(), false, false);
-   public static final TextFileType HPP = new CppFileType("hpp", ".hpp", ICONS.iconHpp(), true, false);
-   public static final TextFileType CPP = new CppFileType("cpp", ".cpp", ICONS.iconCpp(), true, true);
+   public static final TextFileType H = new CppFileType("h", ".h", new ImageResource2x(ICONS.iconH2x()), true, false);
+   public static final TextFileType C = new CppFileType("c", ".c", new ImageResource2x(ICONS.iconC2x()), false, false);
+   public static final TextFileType HPP = new CppFileType("hpp", ".hpp", new ImageResource2x(ICONS.iconHpp2x()), true, false);
+   public static final TextFileType CPP = new CppFileType("cpp", ".cpp", new ImageResource2x(ICONS.iconCpp2x()), true, true);
    
    public static final TextFileType CLOJURE = 
-         new TextFileType("clojure", "Clojure", EditorLanguage.LANG_CLOJURE, ".clj", ICONS.iconClojure(),
+         new TextFileType("clojure", "Clojure", EditorLanguage.LANG_CLOJURE, ".clj", new ImageResource2x(ICONS.iconClojure2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
    
    public static final TextFileType COFFEE = 
-         new TextFileType("coffee", "Coffee", EditorLanguage.LANG_COFFEE, ".coffee", ICONS.iconCoffee(),
+         new TextFileType("coffee", "Coffee", EditorLanguage.LANG_COFFEE, ".coffee", new ImageResource2x(ICONS.iconCoffee2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
    
    public static final TextFileType CSHARP = 
-         new TextFileType("csharp", "C#", EditorLanguage.LANG_CSHARP, ".cs", ICONS.iconCsharp(),
+         new TextFileType("csharp", "C#", EditorLanguage.LANG_CSHARP, ".cs", new ImageResource2x(ICONS.iconCsharp2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
    
    
    public static final TextFileType GITIGNORE = 
-         new TextFileType("gitignore", "Gitignore", EditorLanguage.LANG_GITIGNORE, ".gitignore", ICONS.iconGitignore(),
+         new TextFileType("gitignore", "Gitignore", EditorLanguage.LANG_GITIGNORE, ".gitignore", new ImageResource2x(ICONS.iconGitignore2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
    
    public static final TextFileType GO = 
-         new TextFileType("go", "Go", EditorLanguage.LANG_GO, ".go", ICONS.iconGo(),
+         new TextFileType("go", "Go", EditorLanguage.LANG_GO, ".go", new ImageResource2x(ICONS.iconGo2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
    
    public static final TextFileType GROOVY = 
-         new TextFileType("groovy", "Groovy", EditorLanguage.LANG_GROOVY, ".groovy", ICONS.iconGroovy(),
+         new TextFileType("groovy", "Groovy", EditorLanguage.LANG_GROOVY, ".groovy", new ImageResource2x(ICONS.iconGroovy2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
    
    public static final TextFileType HASKELL = 
-         new TextFileType("haskell", "Haskell", EditorLanguage.LANG_HASKELL, ".haskell", ICONS.iconHaskell(),
+         new TextFileType("haskell", "Haskell", EditorLanguage.LANG_HASKELL, ".haskell", new ImageResource2x(ICONS.iconHaskell2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
    
    public static final TextFileType HAXE = 
-         new TextFileType("haxe", "Haxe", EditorLanguage.LANG_HAXE, ".haxe", ICONS.iconHaxe(),
+         new TextFileType("haxe", "Haxe", EditorLanguage.LANG_HAXE, ".haxe", new ImageResource2x(ICONS.iconHaxe2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
    
    public static final TextFileType JAVA = 
-         new TextFileType("java", "Java", EditorLanguage.LANG_JAVA, ".java", ICONS.iconJava(),
+         new TextFileType("java", "Java", EditorLanguage.LANG_JAVA, ".java", new ImageResource2x(ICONS.iconJava2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
    
    public static final TextFileType JULIA = 
-         new TextFileType("julia", "Julia", EditorLanguage.LANG_JULIA, ".julia", ICONS.iconJulia(),
+         new TextFileType("julia", "Julia", EditorLanguage.LANG_JULIA, ".julia", new ImageResource2x(ICONS.iconJulia2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
    
    public static final TextFileType LISP = 
-         new TextFileType("lisp", "Lisp", EditorLanguage.LANG_LISP, ".lisp", ICONS.iconLisp(),
+         new TextFileType("lisp", "Lisp", EditorLanguage.LANG_LISP, ".lisp", new ImageResource2x(ICONS.iconLisp2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
    
    public static final TextFileType LUA = 
-         new TextFileType("lua", "Lua", EditorLanguage.LANG_LUA, ".lua", ICONS.iconLua(),
+         new TextFileType("lua", "Lua", EditorLanguage.LANG_LUA, ".lua", new ImageResource2x(ICONS.iconLua2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
    
    public static final TextFileType MAKEFILE = 
-         new TextFileType("makefile", "Makefile", EditorLanguage.LANG_MAKEFILE, ".makefile", ICONS.iconMakefile(),
+         new TextFileType("makefile", "Makefile", EditorLanguage.LANG_MAKEFILE, ".makefile", new ImageResource2x(ICONS.iconMakefile2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
    
    public static final TextFileType MATLAB = 
-         new TextFileType("matlab", "Matlab", EditorLanguage.LANG_MATLAB, ".m", ICONS.iconMatlab(),
+         new TextFileType("matlab", "Matlab", EditorLanguage.LANG_MATLAB, ".m", new ImageResource2x(ICONS.iconMatlab2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
    
    public static final TextFileType PERL = 
-         new TextFileType("perl", "Perl", EditorLanguage.LANG_PERL, ".pl", ICONS.iconPerl(),
+         new TextFileType("perl", "Perl", EditorLanguage.LANG_PERL, ".pl", new ImageResource2x(ICONS.iconPerl2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
    
    public static final TextFileType RUBY = 
-         new TextFileType("ruby", "Ruby", EditorLanguage.LANG_RUBY, ".rb", ICONS.iconRuby(),
+         new TextFileType("ruby", "Ruby", EditorLanguage.LANG_RUBY, ".rb", new ImageResource2x(ICONS.iconRuby2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
    
    public static final TextFileType RUST = 
-         new TextFileType("rust", "Rust", EditorLanguage.LANG_RUST, ".rs", ICONS.iconRust(),
+         new TextFileType("rust", "Rust", EditorLanguage.LANG_RUST, ".rs", new ImageResource2x(ICONS.iconRust2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
    
    public static final TextFileType SCALA = 
-         new TextFileType("scala", "Scala", EditorLanguage.LANG_SCALA, ".scala", ICONS.iconScala(),
+         new TextFileType("scala", "Scala", EditorLanguage.LANG_SCALA, ".scala", new ImageResource2x(ICONS.iconScala2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
    
    public static final TextFileType SNIPPETS =
-         new TextFileType("snippets", "Snippets", EditorLanguage.LANG_SNIPPETS, ".snippets", ICONS.iconSnippets(),
+         new TextFileType("snippets", "Snippets", EditorLanguage.LANG_SNIPPETS, ".snippets", new ImageResource2x(ICONS.iconSnippets2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
    
    public static final RDataType RDATA = new RDataType();
+   public static final RDSDataType RDS = new RDSDataType();
    public static final RProjectType RPROJECT = new RProjectType();
 
    public static final DataFrameType DATAFRAME = new DataFrameType();
    public static final UrlContentType URLCONTENT = new UrlContentType();
    public static final CodeBrowserType CODEBROWSER = new CodeBrowserType();
    public static final ProfilerType PROFILER = new ProfilerType();
+   public static final ObjectExplorerFileType OBJECT_EXPLORER = new ObjectExplorerFileType();
 
    public static final BrowserType BROWSER = new BrowserType();
 
@@ -287,131 +295,134 @@ public class FileTypeRegistry
 
       FileIconResources icons = ICONS;
 
-      register("", TEXT, icons.iconText());
-      register("*.txt", TEXT, icons.iconText());
-      register("*.log", TEXT, icons.iconText());
-      register("README", TEXT, icons.iconText());
-      register(".gitignore", TEXT, icons.iconText());
-      register(".Rbuildignore", TEXT, icons.iconText());
-      register("packrat.lock", DCF, icons.iconDCF());
-      register("*.r", R, icons.iconRdoc());
-      register("*.q", R, icons.iconRdoc());
-      register("*.s", R, icons.iconRdoc());
-      register(".Rprofile", R, icons.iconRprofile());
-      register("Rprofile.site", R, icons.iconRprofile());
-      register(".Renviron", SH, icons.iconSh());
-      register("Renviron.site", SH, icons.iconSh());
-      register("DESCRIPTION", DCF, icons.iconDCF());
-      register("INDEX", TEXT, icons.iconText());
-      register("LICENCE", TEXT, icons.iconText());
-      register("MD5", TEXT, icons.iconText());
-      register("NEWS", TEXT, icons.iconText());
-      register("PORTING", TEXT, icons.iconText());
-      register("COPYING", TEXT, icons.iconText());
-      register("COPYING.LIB", TEXT, icons.iconText());
-      register("BUGS", TEXT, icons.iconText());
-      register("CHANGES", TEXT, icons.iconText());
-      register("CHANGELOG", TEXT, icons.iconText());
-      register("INSTALL", SH, icons.iconSh());
-      register("TODO", TEXT, icons.iconText());
-      register("THANKS", TEXT, icons.iconText());
-      register("configure", SH, icons.iconSh());
-      register("configure.win", SH, icons.iconSh());
-      register("cleanup", SH, icons.iconSh());
-      register("cleanup.win", SH, icons.iconSh());
-      register("Makefile", MAKEFILE, icons.iconMakefile());
-      register("Makefile.in", MAKEFILE, icons.iconMakefile());
-      register("Makefile.win", MAKEFILE, icons.iconMakefile());
-      register("Makevars", MAKEFILE, icons.iconMakefile());
-      register("Makevars.in", MAKEFILE, icons.iconMakefile());
-      register("Makevars.win", MAKEFILE, icons.iconMakefile());
-      register("TUTORIAL", DCF, icons.iconDCF());
-      register("NAMESPACE", NAMESPACE, icons.iconText());
-      register("*.rhistory", RHISTORY, icons.iconRhistory());
-      register("*.rproj", RPROJECT, icons.iconRproject());
-      register("*.rnw", SWEAVE, icons.iconRsweave());
-      register("*.rtex", SWEAVE, icons.iconRsweave());
-      register("*.snw", SWEAVE, icons.iconRsweave());
-      register("*.nw", SWEAVE, icons.iconRsweave());
-      register("*.tex", TEX, icons.iconTex());
-      register("*.latex", TEX, icons.iconTex());
-      register("*.sty", TEX, icons.iconTex());
-      register("*.cls", TEX, icons.iconTex());
-      register("*.bbl", TEX, icons.iconTex());
-      register("*.dtx", TEX, icons.iconTex());
-      register("*.ins", TEX, icons.iconTex());
-      register("*.rhtml", RHTML, icons.iconRhtml());
-      register("*.htm", HTML, icons.iconHTML());
-      register("*.html", HTML, icons.iconHTML());
-      register("*.css", CSS, icons.iconCss());
-      register("*.js", JS, icons.iconJavascript());
-      register("*.json", JSON, icons.iconJavascript());
-      register("*.rmd", RMARKDOWN, icons.iconRmarkdown());
-      register("*.rmarkdown", RMARKDOWN, icons.iconRmarkdown());
-      register("*.nb.html", RNOTEBOOK, icons.iconRnotebook());
-      register("*.rpres", RPRESENTATION, icons.iconRpresentation());
-      register("*.md", MARKDOWN, icons.iconMarkdown());
-      register("*.mdtxt", MARKDOWN, icons.iconMarkdown());
-      register("*.markdown*", MARKDOWN, icons.iconMarkdown());
-      register("*.bib", TEXT, icons.iconText());
-      register("*.c", C, icons.iconC());
-      register("*.cpp", CPP, icons.iconCpp());
-      register("*.cc", CPP, icons.iconCpp());
-      register("*.h", H, icons.iconH());
-      register("*.hpp", HPP, icons.iconHpp());
-      register("*.f", TEXT, icons.iconText());
-      register("*.Rout.save", TEXT, icons.iconText());
-      register("*.rd", RD, icons.iconRd());
-      register("*.rdata", RDATA, icons.iconRdata());
-      register("*.rda", RDATA, icons.iconRdata());
-      register("*.Rproj", RPROJECT, icons.iconRproject());
-      register("*.dcf", DCF, icons.iconDCF());
-      register("*.mmd", MERMAID, icons.iconMermaid());
-      register("*.gv", GRAPHVIZ, icons.iconGraphviz());
-      register("*.dot", GRAPHVIZ, icons.iconGraphviz());
-      register("*.py", PYTHON, icons.iconPython());
-      register("*.sql", SQL, icons.iconSql());
-      register("*.sh", SH, icons.iconSh());
-      register("*.yml", YAML, icons.iconYaml());
-      register("*.yaml", YAML, icons.iconYaml());
-      register("*.xml", XML, icons.iconXml());
-      register("*.stan", STAN, icons.iconStan());
+      register("", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("*.txt", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("*.log", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("README", TEXT, new ImageResource2x(icons.iconText2x()));
+      register(".gitignore", TEXT, new ImageResource2x(icons.iconText2x()));
+      register(".Rbuildignore", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("packrat.lock", DCF, new ImageResource2x(icons.iconDCF2x()));
+      register("*.r", R, new ImageResource2x(icons.iconRdoc2x()));
+      register("*.q", R, new ImageResource2x(icons.iconRdoc2x()));
+      register("*.s", R, new ImageResource2x(icons.iconRdoc2x()));
+      register(".Rprofile", R, new ImageResource2x(icons.iconRprofile2x()));
+      register("Rprofile.site", R, new ImageResource2x(icons.iconRprofile2x()));
+      register(".Renviron", SH, new ImageResource2x(icons.iconSh2x()));
+      register("Renviron.site", SH, new ImageResource2x(icons.iconSh2x()));
+      register("DESCRIPTION", DCF, new ImageResource2x(icons.iconDCF2x()));
+      register("INDEX", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("LICENCE", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("MD5", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("NEWS", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("PORTING", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("COPYING", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("COPYING.LIB", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("BUGS", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("CHANGES", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("CHANGELOG", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("INSTALL", SH, new ImageResource2x(icons.iconSh2x()));
+      register("TODO", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("THANKS", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("configure", SH, new ImageResource2x(icons.iconSh2x()));
+      register("configure.win", SH, new ImageResource2x(icons.iconSh2x()));
+      register("cleanup", SH, new ImageResource2x(icons.iconSh2x()));
+      register("cleanup.win", SH, new ImageResource2x(icons.iconSh2x()));
+      register("Makefile", MAKEFILE, new ImageResource2x(icons.iconMakefile2x()));
+      register("Makefile.in", MAKEFILE, new ImageResource2x(icons.iconMakefile2x()));
+      register("Makefile.win", MAKEFILE, new ImageResource2x(icons.iconMakefile2x()));
+      register("Makevars", MAKEFILE, new ImageResource2x(icons.iconMakefile2x()));
+      register("Makevars.in", MAKEFILE, new ImageResource2x(icons.iconMakefile2x()));
+      register("Makevars.win", MAKEFILE, new ImageResource2x(icons.iconMakefile2x()));
+      register("TUTORIAL", DCF, new ImageResource2x(icons.iconDCF2x()));
+      register("NAMESPACE", NAMESPACE, new ImageResource2x(icons.iconText2x()));
+      register("*.rhistory", RHISTORY, new ImageResource2x(icons.iconRhistory2x()));
+      register("*.rproj", RPROJECT, new ImageResource2x(icons.iconRproject2x()));
+      register("*.rnw", SWEAVE, new ImageResource2x(icons.iconRsweave2x()));
+      register("*.rtex", SWEAVE, new ImageResource2x(icons.iconRsweave2x()));
+      register("*.snw", SWEAVE, new ImageResource2x(icons.iconRsweave2x()));
+      register("*.nw", SWEAVE, new ImageResource2x(icons.iconRsweave2x()));
+      register("*.tex", TEX, new ImageResource2x(icons.iconTex2x()));
+      register("*.latex", TEX, new ImageResource2x(icons.iconTex2x()));
+      register("*.sty", TEX, new ImageResource2x(icons.iconTex2x()));
+      register("*.cls", TEX, new ImageResource2x(icons.iconTex2x()));
+      register("*.bbl", TEX, new ImageResource2x(icons.iconTex2x()));
+      register("*.dtx", TEX, new ImageResource2x(icons.iconTex2x()));
+      register("*.ins", TEX, new ImageResource2x(icons.iconTex2x()));
+      register("*.rhtml", RHTML, new ImageResource2x(icons.iconRhtml2x()));
+      register("*.htm", HTML, new ImageResource2x(icons.iconHTML2x()));
+      register("*.html", HTML, new ImageResource2x(icons.iconHTML2x()));
+      register("*.css", CSS, new ImageResource2x(icons.iconCss2x()));
+      register("*.js", JS, new ImageResource2x(icons.iconJavascript2x()));
+      register("*.json", JSON, new ImageResource2x(icons.iconJavascript2x()));
+      register("*.rmd", RMARKDOWN, new ImageResource2x(icons.iconRmarkdown2x()));
+      register("*.rmarkdown", RMARKDOWN, new ImageResource2x(icons.iconRmarkdown2x()));
+      register("*.nb.html", RNOTEBOOK, new ImageResource2x(icons.iconRnotebook2x()));
+      register("*.rpres", RPRESENTATION, new ImageResource2x(icons.iconRpresentation2x()));
+      register("*.md", MARKDOWN, new ImageResource2x(icons.iconMarkdown2x()));
+      register("*.mdtxt", MARKDOWN, new ImageResource2x(icons.iconMarkdown2x()));
+      register("*.markdown*", MARKDOWN, new ImageResource2x(icons.iconMarkdown2x()));
+      register("*.bib", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("*.c", C, new ImageResource2x(icons.iconC2x()));
+      register("*.cpp", CPP, new ImageResource2x(icons.iconCpp2x()));
+      register("*.cc", CPP, new ImageResource2x(icons.iconCpp2x()));
+      register("*.h", H, new ImageResource2x(icons.iconH2x()));
+      register("*.hpp", HPP, new ImageResource2x(icons.iconHpp2x()));
+      register("*.f", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("*.Rout.save", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("*.rd", RD, new ImageResource2x(icons.iconRd2x()));
+      register("*.rdata", RDATA, new ImageResource2x(icons.iconRdata2x()));
+      register("*.rda", RDATA, new ImageResource2x(icons.iconRdata2x()));
+      register("*.rds", RDS, new ImageResource2x(icons.iconRdata2x()));
+      register("*.Rproj", RPROJECT, new ImageResource2x(icons.iconRproject2x()));
+      register("*.dcf", DCF, new ImageResource2x(icons.iconDCF2x()));
+      register("*.mmd", MERMAID, new ImageResource2x(icons.iconMermaid2x()));
+      register("*.gv", GRAPHVIZ, new ImageResource2x(icons.iconGraphviz2x()));
+      register("*.dot", GRAPHVIZ, new ImageResource2x(icons.iconGraphviz2x()));
+      register("*.py", PYTHON, new ImageResource2x(icons.iconPython2x()));
+      register("*.sql", SQL, new ImageResource2x(icons.iconSql2x()));
+      register("*.sh", SH, new ImageResource2x(icons.iconSh2x()));
+      register("*.tml", TOML, new ImageResource2x(icons.iconToml2x()));
+      register("*.toml", TOML, new ImageResource2x(icons.iconToml2x()));
+      register("*.yml", YAML, new ImageResource2x(icons.iconYaml2x()));
+      register("*.yaml", YAML, new ImageResource2x(icons.iconYaml2x()));
+      register("*.xml", XML, new ImageResource2x(icons.iconXml2x()));
+      register("*.stan", STAN, new ImageResource2x(icons.iconStan2x()));
       
-      register("*.clj", CLOJURE, icons.iconClojure());
-      register("*.cloj", CLOJURE, icons.iconClojure());
-      register("*.clojure", CLOJURE, icons.iconClojure());
-      register("*.coffee", COFFEE, icons.iconCoffee());
-      register("*.cs", CSHARP, icons.iconCsharp());
-      register(".gitignore", GITIGNORE, icons.iconGitignore());
-      register("*.go", GO, icons.iconGo());
-      register("*.groovy", GROOVY, icons.iconGroovy());
-      register("*.haskell", HASKELL, icons.iconHaskell());
-      register("*.haxe", HAXE, icons.iconHaxe());
-      register("*.java", JAVA, icons.iconJava());
-      register("*.julia", JULIA, icons.iconJulia());
-      register("*.lisp", LISP, icons.iconLisp());
-      register(".emacs", LISP, icons.iconLisp());
-      register("*.el", LISP, icons.iconLisp());
-      register("*.lua", LUA, icons.iconLua());
-      register("*.m", MATLAB, icons.iconMatlab());
-      register("*.pl", PERL, icons.iconPerl());
-      register("*.rb", RUBY, icons.iconRuby());
-      register("*.rs", RUST, icons.iconRust());
-      register("*.scala", SCALA, icons.iconScala());
-      register("*.snippets", SNIPPETS, icons.iconSnippets());
-      register("*.Rprofvis", PROFILER, icons.iconRprofile());
+      register("*.clj", CLOJURE, new ImageResource2x(icons.iconClojure2x()));
+      register("*.cloj", CLOJURE, new ImageResource2x(icons.iconClojure2x()));
+      register("*.clojure", CLOJURE, new ImageResource2x(icons.iconClojure2x()));
+      register("*.coffee", COFFEE, new ImageResource2x(icons.iconCoffee2x()));
+      register("*.cs", CSHARP, new ImageResource2x(icons.iconCsharp2x()));
+      register(".gitignore", GITIGNORE, new ImageResource2x(icons.iconGitignore2x()));
+      register("*.go", GO, new ImageResource2x(icons.iconGo2x()));
+      register("*.groovy", GROOVY, new ImageResource2x(icons.iconGroovy2x()));
+      register("*.haskell", HASKELL, new ImageResource2x(icons.iconHaskell2x()));
+      register("*.haxe", HAXE, new ImageResource2x(icons.iconHaxe2x()));
+      register("*.java", JAVA, new ImageResource2x(icons.iconJava2x()));
+      register("*.julia", JULIA, new ImageResource2x(icons.iconJulia2x()));
+      register("*.lisp", LISP, new ImageResource2x(icons.iconLisp2x()));
+      register(".emacs", LISP, new ImageResource2x(icons.iconLisp2x()));
+      register("*.el", LISP, new ImageResource2x(icons.iconLisp2x()));
+      register("*.lua", LUA, new ImageResource2x(icons.iconLua2x()));
+      register("*.m", MATLAB, new ImageResource2x(icons.iconMatlab2x()));
+      register("*.pl", PERL, new ImageResource2x(icons.iconPerl2x()));
+      register("*.rb", RUBY, new ImageResource2x(icons.iconRuby2x()));
+      register("*.rs", RUST, new ImageResource2x(icons.iconRust2x()));
+      register("*.scala", SCALA, new ImageResource2x(icons.iconScala2x()));
+      register("*.snippets", SNIPPETS, new ImageResource2x(icons.iconSnippets2x()));
+      register("*.Rprofvis", PROFILER, new ImageResource2x(icons.iconRprofile2x()));
 
-      registerIcon(".jpg", icons.iconPng());
-      registerIcon(".jpeg", icons.iconPng());
-      registerIcon(".gif", icons.iconPng());
-      registerIcon(".bmp", icons.iconPng());
-      registerIcon(".tiff", icons.iconPng());
-      registerIcon(".tif", icons.iconPng());
-      registerIcon(".png", icons.iconPng());
+      registerIcon(".jpg", new ImageResource2x(icons.iconPng2x()));
+      registerIcon(".jpeg", new ImageResource2x(icons.iconPng2x()));
+      registerIcon(".gif", new ImageResource2x(icons.iconPng2x()));
+      registerIcon(".bmp", new ImageResource2x(icons.iconPng2x()));
+      registerIcon(".tiff", new ImageResource2x(icons.iconPng2x()));
+      registerIcon(".tif", new ImageResource2x(icons.iconPng2x()));
+      registerIcon(".png", new ImageResource2x(icons.iconPng2x()));
 
-      registerIcon(".pdf", icons.iconPdf());
-      registerIcon(".csv", icons.iconCsv());
-      registerIcon(".docx", icons.iconWord());
+      registerIcon(".pdf", new ImageResource2x(icons.iconPdf2x()));
+      registerIcon(".csv", new ImageResource2x(icons.iconCsv2x()));
+      registerIcon(".docx", new ImageResource2x(icons.iconWord2x()));
 
       for (FileType fileType : FileType.ALL_FILE_TYPES)
       {
@@ -605,9 +616,9 @@ public class FileTypeRegistry
       if (file.isDirectory())
       {
          if (file.isPublicFolder())
-            return ICONS.iconPublicFolder();
+            return new ImageResource2x(ICONS.iconPublicFolder2x());
          else
-            return ICONS.iconFolder();
+            return new ImageResource2x(ICONS.iconFolder2x());
       }
 
       return getIconForFilename(file.getName());
@@ -623,7 +634,7 @@ public class FileTypeRegistry
       if (icon != null)
          return icon;
 
-      return ICONS.iconText();
+      return new ImageResource2x(ICONS.iconText2x());
    }
 
    private void register(String filespec, FileType fileType, ImageResource icon)
