@@ -1,7 +1,7 @@
 /*
  * RSConnectDeploy.java
  *
- * Copyright (C) 2009-15 by RStudio, Inc.
+ * Copyright (C) 2009-17 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -21,6 +21,7 @@ import org.rstudio.core.client.Debug;
 import org.rstudio.core.client.JsArrayUtil;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.files.FileSystemItem;
+import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.widget.Operation;
 import org.rstudio.core.client.widget.OperationWithInput;
 import org.rstudio.core.client.widget.ProgressIndicator;
@@ -35,6 +36,7 @@ import org.rstudio.studio.client.rsconnect.RSConnect;
 import org.rstudio.studio.client.rsconnect.model.RSConnectAccount;
 import org.rstudio.studio.client.rsconnect.model.RSConnectAppName;
 import org.rstudio.studio.client.rsconnect.model.RSConnectApplicationInfo;
+import org.rstudio.studio.client.rsconnect.model.RSConnectApplicationResult;
 import org.rstudio.studio.client.rsconnect.model.RSConnectDeploymentFiles;
 import org.rstudio.studio.client.rsconnect.model.RSConnectDeploymentRecord;
 import org.rstudio.studio.client.rsconnect.model.RSConnectPublishResult;
@@ -63,10 +65,10 @@ import com.google.gwt.uibinder.client.UiBinder;
 import com.google.gwt.uibinder.client.UiField;
 import com.google.gwt.user.client.Command;
 import com.google.gwt.user.client.ui.Anchor;
-import com.google.gwt.user.client.ui.CheckBox;
 import com.google.gwt.user.client.ui.Composite;
 import com.google.gwt.user.client.ui.Grid;
 import com.google.gwt.user.client.ui.HTMLPanel;
+import com.google.gwt.user.client.ui.HorizontalPanel;
 import com.google.gwt.user.client.ui.Image;
 import com.google.gwt.user.client.ui.InlineLabel;
 import com.google.gwt.user.client.ui.Label;
@@ -87,7 +89,13 @@ public class RSConnectDeploy extends Composite
    public interface DeployStyle extends CssResource
    {
       String accountAnchor();
+      String accountEntry();
       String accountList();
+      String accountListCondensed();
+      String appDetailsPanel();
+      String appErrorMessage();
+      String appErrorPanel();
+      String appWarningIcon();
       String controlLabel();
       String deployLabel();
       String descriptionPanel();
@@ -97,6 +105,7 @@ public class RSConnectDeploy extends Composite
       String launchCheck();
       String normalStatus();
       String otherStatus();
+      String progressPanel();
       String rootCell();
       String source();
       String sourceDestLabels();
@@ -104,18 +113,25 @@ public class RSConnectDeploy extends Composite
       String transferArrow();
       String urlAnchor();
       String wizard();
-      String progressPanel();
-      String appDetailsPanel();
       String wizardDeployPage();
    }
    
    public interface DeployResources extends ClientBundle
-   {  
-      ImageResource publishShinyIllustration();
-      ImageResource publishRmdIllustration();
-      ImageResource publishPlotIllustration();
-      ImageResource publishPresentationIllustration();
-      ImageResource publishHTMLIllustration();
+   {
+      @Source("publishShinyIllustration_2x.png")
+      ImageResource publishShinyIllustration2x();
+
+      @Source("publishRmdIllustration_2x.png")
+      ImageResource publishRmdIllustration2x();
+
+      @Source("publishPlotIllustration_2x.png")
+      ImageResource publishPlotIllustration2x();
+
+      @Source("publishPresentationIllustration_2x.png")
+      ImageResource publishPresentationIllustration2x();
+
+      @Source("publishHTMLIllustration_2x.png")
+      ImageResource publishHTMLIllustration2x();
 
       @Source("RSConnectDeploy.css")
       DeployStyle style();
@@ -205,9 +221,12 @@ public class RSConnectDeploy extends Composite
          @Override
          public void onClick(ClickEvent event)
          {
-            forgetPreviousDeployment();
             event.preventDefault();
             event.stopPropagation();
+            display_.showMessage(GlobalDisplay.MSG_INFO, 
+                  "Create New Content", 
+                  "To publish this content to a new location, click the Publish drop-down menu " +
+                  "and choose Other Destination.");
          }
       });
 
@@ -294,6 +313,9 @@ public class RSConnectDeploy extends Composite
                {
                   accountList_.selectAccount(preferred);
                }
+               
+               // validate the app name now that we have an account selected
+               appName_.validateAppName();
             }
          }
       });
@@ -306,16 +328,14 @@ public class RSConnectDeploy extends Composite
          {
             if (fromPrevious_ != null)
             {
-               boolean existing = accountList_.getSelectedAccount().equals(
-                     fromPrevious_.getAccount());
-               appInfoPanel_.setVisible(existing);
-               newAppPanel_.setVisible(!existing);
-
-               // validate name if necessary
-               if (existing && onDeployEnabled_ != null)
-                  onDeployEnabled_.execute();
-               else if (!existing)
-                  appName_.validateAppName();
+               // re-fetch based on new account (maybe the other account has access)
+               setPreviousInfo();
+            }
+            else
+            {
+               // re-validate app name as different accounts have different 
+               // name restrictions
+               appName_.validateAppName();
             }
          }
       });
@@ -384,6 +404,7 @@ public class RSConnectDeploy extends Composite
    public void setOnDeployDisabled(Command cmd)
    {
       appName_.setOnNameIsInvalid(cmd);
+      onDeployDisabled_ = cmd;
    }
    
    public DeployStyle getStyle()
@@ -456,6 +477,7 @@ public class RSConnectDeploy extends Composite
       return new RSConnectPublishResult(
             appName, 
             isUpdate() ? null : appTitle,
+            isUpdate() ? fromPrevious_.getAppId() : null,
             getSelectedAccount(), 
             source_,
             new RSConnectPublishSettings(deployFiles, 
@@ -477,23 +499,26 @@ public class RSConnectDeploy extends Composite
          return;
       }
       
-      // no need to validate names for updates
       if (isUpdate())
       {
+         // no need to validate names for updates
          onComplete.execute(true);
       }
-      
-      checkForExistingApp(getSelectedAccount(), appName_.getName(), 
-            new OperationWithInput<Boolean>()
-            {
-               @Override
-               public void execute(Boolean valid)
+      else
+      {
+         // see if there's an app with this name before running the deploy
+         checkForExistingApp(getSelectedAccount(), appName_.getName(), 
+               new OperationWithInput<Boolean>()
                {
-                  onComplete.execute(valid);
-                  if (!valid)
-                     focus();
-               }
-            });
+                  @Override
+                  public void execute(Boolean valid)
+                  {
+                     onComplete.execute(valid);
+                     if (!valid)
+                        focus();
+                  }
+               });
+      }
    }
    
    public void setUnsanitizedAppName(String name)
@@ -502,6 +527,15 @@ public class RSConnectDeploy extends Composite
       appName_.validateAppName();
    }
 
+   @Override
+   public boolean supportsTitle()
+   {
+      if (getSelectedAccount() == null)
+         return true;
+      else
+         return !getSelectedAccount().isCloudAccount();
+   }
+
    @Override
    public void generateAppName(String title,
          final CommandWithArg<RSConnectAppName> result)
@@ -536,7 +570,7 @@ public class RSConnectDeploy extends Composite
    private void setFileList(ArrayList<String> files,
          ArrayList<String> additionalFiles, ArrayList<String> ignoredFiles)
    {
-      fileChecks_ = new ArrayList<CheckBox>();
+      fileChecks_ = new ArrayList<DirEntryCheckBox>();
       
       // clear existing file list
       fileListPanel_.clear(); 
@@ -596,7 +630,12 @@ public class RSConnectDeploy extends Composite
    
    private RSConnectAccount getSelectedAccount()
    {
-      return accountList_.getSelectedAccount();
+      if (accountList_.isVisible())
+         return accountList_.getSelectedAccount();
+      else if (accountEntry_.isVisible())
+         return accountEntry_.getAccount();
+      else
+         return null;
    }
    
    private void hideCheckUncheckAllButton()
@@ -615,76 +654,55 @@ public class RSConnectDeploy extends Composite
          appExistingName_.setText(fromPrevious_.getDisplayName());
          appProgressPanel_.setVisible(true);
          appInfoPanel_.setVisible(true);
-
-         final ServerRequestCallback<JsArray<RSConnectApplicationInfo>> 
-               onAppsReceived = 
-               new ServerRequestCallback<JsArray<RSConnectApplicationInfo>>()
-         {
-            @Override
-            public void onResponseReceived(
-                  JsArray<RSConnectApplicationInfo> infos)
-            {
-               // hide server progress
-               appProgressPanel_.setVisible(false);
-
-               // find an app with the same account, server, and name;
-               // when found, populate the UI with app details
-               boolean found = false;
-               if (infos != null)
-               {
-                  for (int i = 0; i < infos.length(); i++)
-                  {
-                     RSConnectApplicationInfo info = infos.get(i);
-                     if (info.getName() == fromPrevious_.getName())
-                     {
-                        showAppInfo(info);
-                        found = true;
-                        break;
-                     }
-                  }
-               }
-
-               if (!found)
-               {
-                  forgetPreviousDeployment();
-               }
-            }
-            @Override
-            public void onError(ServerError error)
-            {
-               // it's okay if we fail here, since the application info
-               // display is purely informative
-               appProgressPanel_.setVisible(false);
-               showAppInfo(null);
-            }
-         };
-
+         appErrorPanel_.setVisible(false);
          
+         // default to details supplied by record, but use selected account instead if we know it
+         String accountName = fromPrevious_.getAccountName();
+         String server = fromPrevious_.getServer();
+         RSConnectAccount account = accountList_.getSelectedAccount();
+         if (account != null)
+         {
+            accountName = account.getName();
+            server = account.getServer();
+         }
+
          if (!StringUtil.isNullOrEmpty(fromPrevious_.getAppId()))
          {
             // we know this app's ID, so get its details directly
             server_.getRSConnectApp(
                   fromPrevious_.getAppId(),
-                  fromPrevious_.getAccountName(), 
-                  fromPrevious_.getServer(), 
-                  new ServerRequestCallback<RSConnectApplicationInfo>()
+                  accountName,
+                  server,
+                  fromPrevious_.getHostUrl(),
+                  new ServerRequestCallback<RSConnectApplicationResult>()
                   {
                      @Override
                      public void onResponseReceived(
-                           RSConnectApplicationInfo info)
+                           RSConnectApplicationResult info)
                      {
                         // create a single-entry array with the app ID 
                         JsArray<RSConnectApplicationInfo> infos = 
                               JsArray.createArray().cast();
-                        if (info != null)
-                           infos.push(info);
-                        onAppsReceived.onResponseReceived(infos);
+                        if (info.getApp() != null)
+                        {
+                           infos.push(info.getApp());
+                           onAppsReceived_.onResponseReceived(infos);
+                        }
+                        else if (!StringUtil.isNullOrEmpty(info.getError()))
+                        {
+                           onAppsReceived_.onError(info.getError());
+                        }
+                        else
+                        {
+                           onAppsReceived_.onError("Error retrieving application " + 
+                             fromPrevious_.getAppId() + ".");
+                        }
                      }
 
                      @Override
                      public void onError(ServerError error)
                      {
-                        onAppsReceived.onError(error);
+                        onAppsReceived_.onError(error);
                      }
                   });
          }
@@ -694,11 +712,89 @@ public class RSConnectDeploy extends Composite
             server_.getRSConnectAppList(
                   fromPrevious_.getAccountName(), 
                   fromPrevious_.getServer(), 
-                  onAppsReceived);
+                  onAppsReceived_);
             }
       }
    }
    
+   private void setSingleAccount(RSConnectAccount account)
+   {
+      accountEntry_.setAccount(account);
+      accountEntry_.setVisible(true);
+   }
+   
+   private void populateAccountList(final ProgressIndicator indicator,
+         final boolean isRetry,
+         JsArray<RSConnectAccount> accounts)
+   {
+      // if we're updating a deployment, check to see if there's a match
+      if (fromPrevious_ != null)
+      {
+         boolean matched = false;
+         JsArray<RSConnectAccount> serverList = JsArray.createArray().cast();
+         for (int i = 0; i < accounts.length(); i++)
+         {
+            if (accounts.get(i).equals(fromPrevious_.getAccount()))
+            {
+               // show just this account, and hide the account list
+               setSingleAccount(accounts.get(i));
+               accountList_.setVisible(false);
+               publishFromPanel_.setVisible(false);
+               
+               // populate app info
+               matched = true;
+               break;
+            }
+            
+            // if we haven't found a match yet, remember the account if it matches the server
+            // used to deploy the app
+            if (fromPrevious_.getServer() == accounts.get(i).getServer())
+            {
+               serverList.push(accounts.get(i));
+            }
+         }
+         
+         if (!matched)
+         {
+            // we're updating a deployment, but we don't have a corresponding account
+            setSingleAccount(fromPrevious_.getAccount());
+            publishToLabel_.setStyleName(style_.controlLabel());
+            accountList_.addStyleName(style_.accountListCondensed());
+
+            // filter to only those matching the given server
+            accountList_.setAccountList(serverList);
+         }
+         setPreviousInfo();
+         return;
+      }
+
+      // populate the accounts in the UI (the account display widget filters
+      // based on account criteria)
+      accountEntry_.setVisible(false);
+      publishToLabel_.setVisible(false);
+      int numAccounts = setAccountList(accounts);
+
+      // if this is our first try, ask the user to connect an account
+      // since none are currently connected
+      if (numAccounts == 0 && !isRetry)
+      {
+         connector_.showAccountWizard(accounts.length() == 0, 
+               source_.isShiny(),
+               new OperationWithInput<Boolean>() 
+         {
+            @Override
+            public void execute(Boolean input)
+            {
+               populateAccountList(indicator, true);
+            }
+         });
+      }
+      else
+      {
+         setPreviousInfo();
+      }
+   }
+   
    private void populateAccountList(final ProgressIndicator indicator,
                                     final boolean isRetry)
    {
@@ -708,29 +804,7 @@ public class RSConnectDeploy extends Composite
          @Override
          public void onResponseReceived(JsArray<RSConnectAccount> accounts)
          {
-            // populate the accounts in the UI (the account display widget 
-            // filters based on account criteria)
-            int numAccounts = setAccountList(accounts);
-
-            // if this is our first try, ask the user to connect an account
-            // since none are currently connected
-            if (numAccounts == 0 && !isRetry)
-            {
-               connector_.showAccountWizard(accounts.length() == 0, 
-                     source_.isShiny(),
-                     new OperationWithInput<Boolean>() 
-               {
-                  @Override
-                  public void execute(Boolean input)
-                  {
-                     populateAccountList(indicator, true);
-                  }
-               });
-            }
-            else
-            {
-               setPreviousInfo();
-            }
+            populateAccountList(indicator, isRetry, accounts);
          }
          
          @Override
@@ -838,7 +912,7 @@ public class RSConnectDeploy extends Composite
 
    private void addFile(String path, boolean checked)
    {
-      CheckBox fileCheck = new CheckBox(path);
+      DirEntryCheckBox fileCheck = new DirEntryCheckBox(path);
       fileCheck.setValue(checked);
       fileListPanel_.add(fileCheck);
       fileChecks_.add(fileCheck);
@@ -853,7 +927,7 @@ public class RSConnectDeploy extends Composite
       {
          if (fileChecks_.get(i).getValue() == checked)
          {
-            files.add(fileChecks_.get(i).getText());
+            files.add(fileChecks_.get(i).getPath());
          }
       }
       return files;
@@ -914,8 +988,8 @@ public class RSConnectDeploy extends Composite
 
       for (int i = 0; i < fileChecks_.size(); i++)
       {
-         CheckBox fileCheck = fileChecks_.get(i);
-         if (fileCheck.getText().equals(path))
+         DirEntryCheckBox fileCheck = fileChecks_.get(i);
+         if (fileCheck.getPath().equals(path))
          {
             // don't allow the user to unselect the primary file
             fileCheck.setEnabled(false);
@@ -941,71 +1015,68 @@ public class RSConnectDeploy extends Composite
              contentType_ == RSConnect.CONTENT_TYPE_HTML)
          {
             descriptionImage_.setResource(
-                  RSConnectResources.INSTANCE.previewPlot());
+                  new ImageResource2x(RSConnectResources.INSTANCE.previewPlot2x()));
          }
          else if (contentType_ == RSConnect.CONTENT_TYPE_PRES)
          {
             descriptionImage_.setResource(
-                  RSConnectResources.INSTANCE.previewPresentation());
+                  new ImageResource2x(RSConnectResources.INSTANCE.previewPresentation2x()));
          }
          else
          {
             descriptionImage_.setResource(
-                     RSConnectResources.INSTANCE.previewDoc());
+                  new ImageResource2x(RSConnectResources.INSTANCE.previewDoc2x()));
          }
       }
       
       // if the app name textbox isn't populated, derive from the filename
       // (for apps and documents--other content types use temporary filenames)
-      if (appName_.getTitle().isEmpty() && 
-            (contentType_ == RSConnect.CONTENT_TYPE_APP || 
+      if (appName_.getTitle().isEmpty())
+      {
+         if (contentType_ == RSConnect.CONTENT_TYPE_APP || 
              contentType_ == RSConnect.CONTENT_TYPE_APP_SINGLE || 
-             contentType_ == RSConnect.CONTENT_TYPE_DOCUMENT))
-      {
-         // set the app name to the filename
-         String appTitle = 
-               FilePathUtils.fileNameSansExtension(source_.getSourceFile());
-
-         // if this is a document with the name "index", guess the directory
-         // as the content name rather than the file
-         if (contentType_ == RSConnect.CONTENT_TYPE_DOCUMENT &&
-             appTitle.toLowerCase().equals("index"))
+             contentType_ == RSConnect.CONTENT_TYPE_DOCUMENT)
          {
-            appTitle = FilePathUtils.fileNameSansExtension(
-                  source_.getDeployDir());
-         }
+            // set the app name to the filename
+            String appTitle = 
+                  FilePathUtils.fileNameSansExtension(source_.getSourceFile());
 
-         setUnsanitizedAppName(appTitle);
-      }
-      
-      // for websites, use the directory name
-      if (appName_.getTitle().isEmpty() && 
-          contentType_ == RSConnect.CONTENT_TYPE_WEBSITE)
-      {
-         setUnsanitizedAppName(FilePathUtils.fileNameSansExtension(
-               source_.getWebsiteDir()));
+            // if this is a document with the name "index", guess the directory
+            // as the content name rather than the file
+            if (contentType_ == RSConnect.CONTENT_TYPE_DOCUMENT &&
+                appTitle.toLowerCase().equals("index"))
+            {
+               appTitle = FilePathUtils.fileNameSansExtension(
+                     source_.getDeployDir());
+            }
+
+            setUnsanitizedAppName(appTitle);
+         }
+         else if (contentType_ == RSConnect.CONTENT_TYPE_WEBSITE)
+         {
+            setUnsanitizedAppName(FilePathUtils.fileNameSansExtension(
+                  source_.getWebsiteDir()));
+         }
       }
       
       ImageResource illustration = null;
       if (contentType_ == RSConnect.CONTENT_TYPE_APP)
-         illustration = RESOURCES.publishShinyIllustration();
+         illustration = new ImageResource2x(RESOURCES.publishShinyIllustration2x());
       else if (contentType_ == RSConnect.CONTENT_TYPE_PLOT)
-         illustration = RESOURCES.publishPlotIllustration();
+         illustration = new ImageResource2x(RESOURCES.publishPlotIllustration2x());
       else if (contentType_ == RSConnect.CONTENT_TYPE_DOCUMENT)
-         illustration = RESOURCES.publishRmdIllustration();
+         illustration = new ImageResource2x(RESOURCES.publishRmdIllustration2x());
       else if (contentType_ == RSConnect.CONTENT_TYPE_HTML)
-         illustration = RESOURCES.publishHTMLIllustration();
+         illustration = new ImageResource2x(RESOURCES.publishHTMLIllustration2x());
       else if (contentType_ == RSConnect.CONTENT_TYPE_PRES)
-         illustration = RESOURCES.publishPresentationIllustration();
+         illustration = new ImageResource2x(RESOURCES.publishPresentationIllustration2x());
       if (illustration != null)
          deployIllustration_.setResource(illustration);
    }
    
    private boolean isUpdate()
    {
-      return fromPrevious_ != null && 
-            getSelectedAccount() != null &&
-            getSelectedAccount().equals(fromPrevious_.getAccount());
+      return fromPrevious_ != null;
    }
    
    private void checkForExistingApp(final RSConnectAccount account, 
@@ -1097,10 +1168,10 @@ public class RSConnectDeploy extends Composite
    private void checkUncheckAll()
    {
       allChecked_ = !allChecked_;
-      for (CheckBox box: fileChecks_)
+      for (DirEntryCheckBox box: fileChecks_)
       {
          // don't toggle state for disabled boxes, or common Shiny .R filenames
-         String file = box.getText().toLowerCase();
+         String file = box.getPath().toLowerCase();
          if (box.isEnabled() &&
              file != "ui.r" &&
              file != "server.r" &&
@@ -1112,7 +1183,70 @@ public class RSConnectDeploy extends Composite
       checkUncheckAllButton_.setText(allChecked_ ? "Uncheck All" : "Check All");
    }
    
+   private class OnAppsReceived extends ServerRequestCallback<JsArray<RSConnectApplicationInfo>>
+   {
+      @Override
+      public void onResponseReceived(
+            JsArray<RSConnectApplicationInfo> infos)
+      {
+         // hide server progress
+         appProgressPanel_.setVisible(false);
+
+         // find an app with the same account, server, and name; when found,
+         // populate the UI with app details
+         boolean found = false;
+         if (infos != null)
+         {
+            for (int i = 0; i < infos.length(); i++)
+            {
+               RSConnectApplicationInfo info = infos.get(i);
+               if (info.getName() == fromPrevious_.getName())
+               {
+                  showAppInfo(info);
+                  found = true;
+                  break;
+               }
+            }
+         }
+
+         if (!found)
+         {
+            forgetPreviousDeployment();
+         }
+      }
+      
+      public void onError(String error)
+      {
+         // it's okay if we fail here, since the application info display is
+         // purely informative
+         appProgressPanel_.setVisible(false);
+         showAppInfo(null);
+         if (!StringUtil.isNullOrEmpty(error))
+         {
+            showAppError(error);
+            if (onDeployDisabled_ != null)
+               onDeployDisabled_.execute();
+         }
+      }
+
+      @Override
+      public void onError(ServerError error)
+      {
+         onError(error.getMessage());
+      }
+   };
    
+   private void showAppError(String error)
+   {
+      appErrorPanel_.setVisible(true);
+      appDetailsPanel_.setVisible(false);
+      String[] lines = error.split("\n");
+      
+      // show just the last line, but show all the content on hover
+      appErrorMessage_.setText(lines[lines.length - 1]);
+      appErrorMessage_.setTitle(error);
+   }
+
    @UiField Anchor addAccountAnchor_;
    @UiField Anchor createNewAnchor_;
    @UiField Anchor urlAnchor_;
@@ -1122,24 +1256,29 @@ public class RSConnectDeploy extends Composite
    @UiField HTMLPanel appProgressPanel_;
    @UiField HTMLPanel newAppPanel_;
    @UiField HTMLPanel rootPanel_;
+   @UiField HTMLPanel appErrorPanel_;
    @UiField Image deployIllustration_;
    @UiField Image descriptionImage_;
    @UiField InlineLabel deployLabel_;
+   @UiField Label appErrorMessage_;
    @UiField Label appExistingName_;
    @UiField Label appProgressName_;
    @UiField Label nameLabel_;
+   @UiField Label publishToLabel_;
    @UiField ThemedButton addFileButton_;
    @UiField ThemedButton checkUncheckAllButton_;
    @UiField ThemedButton previewButton_;
    @UiField VerticalPanel fileListPanel_;
    @UiField VerticalPanel filePanel_;
    @UiField VerticalPanel descriptionPanel_;
+   @UiField HorizontalPanel publishFromPanel_;
+   @UiField RSConnectAccountEntry accountEntry_;
    
    // provided fields
    @UiField(provided=true) RSConnectAccountList accountList_;
    @UiField(provided=true) AppNameTextbox appName_;
    
-   private ArrayList<CheckBox> fileChecks_;
+   private ArrayList<DirEntryCheckBox> fileChecks_;
    private ArrayList<String> filesAddedManually_ = 
          new ArrayList<String>();
    
@@ -1153,9 +1292,12 @@ public class RSConnectDeploy extends Composite
    private boolean asStatic_;
    private int contentType_;
    private Command onDeployEnabled_;
+   private Command onDeployDisabled_;
    private RSConnectDeploymentRecord fromPrevious_;
    private boolean allChecked_ = true;
 
+   private final OnAppsReceived onAppsReceived_ = new OnAppsReceived();
+
    private final DeployStyle style_;
    private final boolean forDocument_;
 }
