@@ -26,6 +26,7 @@ import org.rstudio.core.client.TimeBufferedCommand;
 import org.rstudio.core.client.command.CommandBinder;
 import org.rstudio.core.client.command.Handler;
 import org.rstudio.core.client.files.FileSystemItem;
+import org.rstudio.core.client.files.filedialog.events.OpenFileDialogEvent;
 import org.rstudio.core.client.widget.ModifyKeyboardShortcutsWidget;
 import org.rstudio.core.client.widget.Operation;
 import org.rstudio.core.client.widget.OperationWithInput;
@@ -40,15 +41,19 @@ import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.common.GlobalDisplay.NewWindowOptions;
 import org.rstudio.studio.client.common.GlobalProgressDelayer;
 import org.rstudio.studio.client.common.SimpleRequestCallback;
-import org.rstudio.studio.client.common.console.ConsoleProcess;
 import org.rstudio.studio.client.common.dependencies.DependencyManager;
 import org.rstudio.studio.client.common.filetypes.FileTypeRegistry;
 import org.rstudio.studio.client.common.vcs.AskPassManager;
 import org.rstudio.studio.client.common.vcs.ShowPublicKeyDialog;
 import org.rstudio.studio.client.common.vcs.VCSConstants;
 import org.rstudio.studio.client.htmlpreview.HTMLPreview;
+import org.rstudio.studio.client.htmlpreview.events.ShowHTMLPreviewEvent;
+import org.rstudio.studio.client.htmlpreview.events.ShowPageViewerEvent;
+import org.rstudio.studio.client.htmlpreview.events.ShowPageViewerHandler;
+import org.rstudio.studio.client.htmlpreview.model.HTMLPreviewParams;
 import org.rstudio.studio.client.pdfviewer.PDFViewer;
 import org.rstudio.studio.client.projects.ProjectOpener;
+import org.rstudio.studio.client.projects.model.ProjectTemplateRegistryProvider;
 import org.rstudio.studio.client.rmarkdown.RmdOutput;
 import org.rstudio.studio.client.rmarkdown.events.ShinyGadgetDialogEvent;
 import org.rstudio.studio.client.server.Server;
@@ -66,7 +71,8 @@ import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
 import org.rstudio.studio.client.workbench.views.choosefile.ChooseFile;
 import org.rstudio.studio.client.workbench.views.files.events.DirectoryNavigateEvent;
 import org.rstudio.studio.client.workbench.views.source.editors.profiler.ProfilerPresenter;
-import org.rstudio.studio.client.workbench.views.vcs.common.ConsoleProgressDialog;
+import org.rstudio.studio.client.workbench.views.terminal.events.ActivateNamedTerminalEvent;
+import org.rstudio.studio.client.workbench.views.terminal.events.CreateTerminalEvent;
 import org.rstudio.studio.client.workbench.views.vcs.common.events.VcsRefreshEvent;
 import org.rstudio.studio.client.workbench.views.vcs.common.events.VcsRefreshHandler;
 import org.rstudio.studio.client.workbench.views.vcs.git.model.GitState;
@@ -81,7 +87,10 @@ public class Workbench implements BusyHandler,
                                   WorkbenchMetricsChangedHandler,
                                   InstallRtoolsEvent.Handler,
                                   ShinyGadgetDialogEvent.Handler,
-                                  ExecuteUserCommandEvent.Handler
+                                  ExecuteUserCommandEvent.Handler,
+                                  AdminNotificationHandler,
+                                  OpenFileDialogEvent.Handler,
+                                  ShowPageViewerHandler
 {
    interface Binder extends CommandBinder<Commands, Workbench> {}
    
@@ -101,15 +110,16 @@ public class Workbench implements BusyHandler,
                     WorkbenchNewSession newSession,
                     ProjectOpener projectOpener,
                     Provider<GitState> pGitState,
-                    ChooseFile chooseFile,   // required to force gin to create
-                    AskPassManager askPass,  // required to force gin to create
-                    PDFViewer pdfViewer,     // required to force gin to create
-                    HTMLPreview htmlPreview, // required to force gin to create
-                    ProfilerPresenter prof,  // required to force gin to create
-                    ShinyApplication sApp,   // required to force gin to create
-                    DependencyManager dm,    // required to force gin to create
-                    ApplicationVisibility av,// required to force gin to create
-                    RmdOutput rmdOutput)     // required to force gin to create    
+                    ChooseFile chooseFile,                    // force gin to create
+                    AskPassManager askPass,                   // force gin to create
+                    PDFViewer pdfViewer,                      // force gin to create
+                    HTMLPreview htmlPreview,                  // force gin to create
+                    ProfilerPresenter prof,                   // force gin to create
+                    ShinyApplication sApp,                    // force gin to create
+                    DependencyManager dm,                     // force gin to create
+                    ApplicationVisibility av,                 // force gin to create
+                    RmdOutput rmdOutput,                      // force gin to create    
+                    ProjectTemplateRegistryProvider provider) // force gin to create
   {
       view_ = view;
       workbenchContext_ = workbenchContext;
@@ -141,6 +151,9 @@ public class Workbench implements BusyHandler,
       eventBus.addHandler(InstallRtoolsEvent.TYPE, this);
       eventBus.addHandler(ShinyGadgetDialogEvent.TYPE, this);
       eventBus.addHandler(ExecuteUserCommandEvent.TYPE, this);
+      eventBus.addHandler(AdminNotificationEvent.TYPE, this);
+      eventBus.addHandler(OpenFileDialogEvent.TYPE, this);
+      eventBus.addHandler(ShowPageViewerEvent.TYPE, this);
 
       // We don't want to send setWorkbenchMetrics more than once per 1/2-second
       metricsChangedCommand_ = new TimeBufferedCommand(-1, -1, 500)
@@ -385,34 +398,23 @@ public class Workbench implements BusyHandler,
             public void onResponseReceived(TerminalOptions options)
             {
                Desktop.getFrame().openTerminal(options.getTerminalPath(),
-                                               options.getWorkingDirectory(),
-                                               options.getExtraPathEntries());
+                     options.getWorkingDirectory(),
+                     options.getExtraPathEntries());
             }
          });
       }
       else
       {
-         final ProgressIndicator indicator = new GlobalProgressDelayer(
-               globalDisplay_, 500, "Starting shell...").getIndicator();
-         
-         server_.startShellDialog(new ServerRequestCallback<ConsoleProcess>() {
-
-            @Override
-            public void onResponseReceived(ConsoleProcess proc)
-            {
-               indicator.onCompleted();
-               new ConsoleProgressDialog(proc, server_).showModal();
-            }
-            
-            @Override
-            public void onError(ServerError error)
-            {
-               indicator.onError(error.getUserMessage());
-            }
-         });
+         eventBus_.fireEvent(new ActivateNamedTerminalEvent());
       }
    }
    
+   @Handler
+   public void onNewTerminal()
+   {
+      eventBus_.fireEvent(new CreateTerminalEvent());
+   }
+      
    @Handler
    public void onBrowseAddins()
    {
@@ -503,6 +505,93 @@ public class Workbench implements BusyHandler,
       };
    }
    
+   public void onAdminNotification(AdminNotificationEvent event)
+   {
+      AdminNotification notification = event.getAdminNotification();
+      
+      // show dialog
+      globalDisplay_.showMessage(notification.getType(), 
+                                 "Admin Notification", 
+                                 notification.getMessage(),
+                                 adminNotificationAcknowledged(notification.getId()));
+   }
+   
+   @Override
+   public void onOpenFileDialog(OpenFileDialogEvent event)
+   {
+      final ProgressOperationWithInput<FileSystemItem> onSelected =
+            new ProgressOperationWithInput<FileSystemItem>()
+      {
+         @Override
+         public void execute(FileSystemItem input,
+                             ProgressIndicator indicator)
+         {
+            indicator.onCompleted();
+            
+            server_.openFileDialogCompleted(
+                  input == null ? "" : input.getPath(),
+                  new VoidServerRequestCallback());
+         }
+      };
+      
+      String caption = event.getCaption();
+      String label = event.getLabel();
+      int type = event.getType();
+      FileSystemItem initialFilePath = event.getFile();
+      String filter = event.getFilter();
+      boolean selectExisting = event.selectExisting();
+      
+      if (type == OpenFileDialogEvent.TYPE_SELECT_FILE)
+      {
+         if (selectExisting)
+         {
+            fileDialogs_.openFile(
+                  caption,
+                  label,
+                  fsContext_,
+                  initialFilePath,
+                  filter,
+                  false,
+                  onSelected);
+         }
+         else
+         {
+            fileDialogs_.saveFile(
+                  caption,
+                  label,
+                  fsContext_,
+                  initialFilePath,
+                  "",
+                  false,
+                  onSelected);
+         }
+      }
+      else if (type == OpenFileDialogEvent.TYPE_SELECT_DIRECTORY)
+      {
+         fileDialogs_.chooseFolder(
+               caption,
+               label,
+               fsContext_,
+               initialFilePath,
+               onSelected);
+      }
+      else
+      {
+         assert false: "unexpected file dialog type '" + type + "'";
+         server_.openFileDialogCompleted(null, new VoidServerRequestCallback());
+      }
+   }
+   
+   private Operation adminNotificationAcknowledged(final String id)
+   {
+      return new Operation() {
+         public void execute()
+         {
+            server_.adminNotificationAcknowledged(id, new SimpleRequestCallback<Void>());
+         }
+      };
+   }
+   
    @Override
    public void onInstallRtools(final InstallRtoolsEvent event)
    {
@@ -513,6 +602,16 @@ public class Workbench implements BusyHandler,
       }
    }
    
+   @Override
+   public void onShowPageViewer(ShowPageViewerEvent event) 
+   {   
+      // show the page viewer window
+      HTMLPreviewParams params = event.getParams();
+      eventBus_.fireEvent(new ShowHTMLPreviewEvent(params)); 
+      
+      // server will now take care of sending the html_preview_completed event
+   }
+   
    @Override
    public void onExecuteUserCommand(ExecuteUserCommandEvent event)
    {
@@ -536,6 +635,5 @@ public class Workbench implements BusyHandler,
    private final TimeBufferedCommand metricsChangedCommand_;
    private WorkbenchMetrics lastWorkbenchMetrics_;
    private WorkbenchNewSession newSession_;
-   private boolean nearQuotaWarningShown_ = false;
-   
+   private boolean nearQuotaWarningShown_ = false; 
 }
