@@ -1,7 +1,7 @@
 /*
  * Commands.java
  *
- * Copyright (C) 2009-15 by RStudio, Inc.
+ * Copyright (C) 2009-17 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -60,6 +60,9 @@ public abstract class
    public abstract AppCommand executeCurrentFunction();
    public abstract AppCommand executeCurrentSection();
    public abstract AppCommand executeLastCode();
+   public abstract AppCommand executeCurrentLine();
+   public abstract AppCommand executeCurrentStatement();
+   public abstract AppCommand executeCurrentParagraph();
    public abstract AppCommand insertChunk();
    public abstract AppCommand insertChunkR();
    public abstract AppCommand insertChunkBash();
@@ -86,12 +89,15 @@ public abstract class
    public abstract AppCommand markdownHelp();
    public abstract AppCommand openRStudioIDECheatSheet();
    public abstract AppCommand openDataVisualizationCheatSheet();
+   public abstract AppCommand openDataImportCheatSheet();
    public abstract AppCommand openPackageDevelopmentCheatSheet();
    public abstract AppCommand openDataWranglingCheatSheet();
+   public abstract AppCommand openDataTransformationCheatSheet();
    public abstract AppCommand openRMarkdownCheatSheet();
    public abstract AppCommand openRMarkdownReferenceGuide();
    public abstract AppCommand openShinyCheatSheet();
    public abstract AppCommand openRoxygenQuickReference();
+   public abstract AppCommand openSparklyrCheatSheet();
    public abstract AppCommand knitDocument();
    public abstract AppCommand previewHTML();
    public abstract AppCommand publishHTML();
@@ -122,6 +128,8 @@ public abstract class
    public abstract AppCommand jumpToMatching();
    public abstract AppCommand selectToMatching();
    public abstract AppCommand expandToMatching();
+   public abstract AppCommand addCursorAbove();
+   public abstract AppCommand addCursorBelow();
    public abstract AppCommand splitIntoLines();
    public abstract AppCommand toggleDocumentOutline();
    public abstract AppCommand expandSelection();
@@ -159,9 +167,12 @@ public abstract class
    public abstract AppCommand pasteLastYank();
    public abstract AppCommand insertAssignmentOperator();
    public abstract AppCommand insertPipeOperator();
+   public abstract AppCommand openNextFileOnFilesystem();
+   public abstract AppCommand openPreviousFileOnFilesystem();
  
    // Projects
    public abstract AppCommand newProject();
+   public abstract AppCommand newProjectFromTemplate();
    public abstract AppCommand openProject();
    public abstract AppCommand openProjectInNewWindow();
    public abstract AppCommand shareProject();
@@ -195,12 +206,16 @@ public abstract class
    public abstract AppCommand restartRRunAllChunks();
    public abstract AppCommand terminateR();
    public abstract AppCommand activateConsole();
+   public abstract AppCommand activateConsolePane();
    public abstract AppCommand layoutZoomConsole();
+   public abstract AppCommand layoutZoomConsolePane();
+   public abstract AppCommand activateConsolePanePane();
 
    // Files
    public abstract AppCommand newFolder();
    public abstract AppCommand uploadFile();
    public abstract AppCommand copyFile();
+   public abstract AppCommand copyFileTo();
    public abstract AppCommand moveFiles();
    public abstract AppCommand exportFiles();
    public abstract AppCommand renameFile();
@@ -302,7 +317,9 @@ public abstract class
    public abstract AppCommand loadWorkspace();
    public abstract AppCommand importDatasetFromFile();
    public abstract AppCommand importDatasetFromURL();
-   public abstract AppCommand importDatasetFromCSV();
+   public abstract AppCommand importDatasetFromCsv();
+   public abstract AppCommand importDatasetFromCsvUsingReadr();
+   public abstract AppCommand importDatasetFromCsvUsingBase();
    public abstract AppCommand importDatasetFromSAV();
    public abstract AppCommand importDatasetFromSAS();
    public abstract AppCommand importDatasetFromStata();
@@ -345,6 +362,7 @@ public abstract class
    public abstract AppCommand packratBundle();
    public abstract AppCommand packratHelp();
    public abstract AppCommand packratClean();
+   public abstract AppCommand packratCheckStatus();
 
    // Version control
    public abstract AppCommand versionControlHelp();
@@ -367,7 +385,20 @@ public abstract class
    public abstract AppCommand macPreferences();
    public abstract AppCommand showOptions();
    public abstract AppCommand modifyKeyboardShortcuts();
-
+   
+   // Terminal
+   public abstract AppCommand newTerminal();
+   public abstract AppCommand activateTerminal();
+   public abstract AppCommand renameTerminal();
+   public abstract AppCommand closeTerminal();
+   public abstract AppCommand clearTerminalScrollbackBuffer();
+   public abstract AppCommand previousTerminal();
+   public abstract AppCommand nextTerminal();
+   public abstract AppCommand showTerminalInfo();
+   public abstract AppCommand interruptTerminal();
+   public abstract AppCommand sendTerminalToEditor();
+   public abstract AppCommand sendToTerminal();
+    
    // Help
    public abstract AppCommand helpBack();
    public abstract AppCommand helpForward();
@@ -439,9 +470,6 @@ public abstract class
    public abstract AppCommand removeConnection();
    public abstract AppCommand disconnectConnection();
    public abstract AppCommand refreshConnection();
-   public abstract AppCommand sparkLog();
-   public abstract AppCommand sparkUI();
-   public abstract AppCommand sparkHelp();
    
    // Clipboard placeholders
    public abstract AppCommand undoDummy();
@@ -494,22 +522,6 @@ public abstract class
    public abstract AppCommand rsconnectManageAccounts();
    
    // Addins
-   public abstract AppCommand addinsMru0();
-   public abstract AppCommand addinsMru1();
-   public abstract AppCommand addinsMru2();
-   public abstract AppCommand addinsMru3();
-   public abstract AppCommand addinsMru4();
-   public abstract AppCommand addinsMru5();
-   public abstract AppCommand addinsMru6();
-   public abstract AppCommand addinsMru7();
-   public abstract AppCommand addinsMru8();
-   public abstract AppCommand addinsMru9();
-   public abstract AppCommand addinsMru10();
-   public abstract AppCommand addinsMru11();
-   public abstract AppCommand addinsMru12();
-   public abstract AppCommand addinsMru13();
-   public abstract AppCommand addinsMru14();
-   public abstract AppCommand clearAddinsMruList();
    public abstract AppCommand browseAddins();
 
    // Other
