@@ -22,7 +22,6 @@ import com.google.gwt.event.dom.client.HasClickHandlers;
 import com.google.gwt.event.logical.shared.ValueChangeEvent;
 import com.google.gwt.event.logical.shared.ValueChangeHandler;
 import com.google.gwt.event.shared.HandlerRegistration;
-import com.google.gwt.user.client.Command;
 import com.google.inject.Inject;
 
 import org.rstudio.core.client.ListUtil;
@@ -33,14 +32,13 @@ import org.rstudio.core.client.dom.DomUtils;
 import org.rstudio.core.client.js.JsObject;
 import org.rstudio.core.client.widget.MessageDialog;
 import org.rstudio.core.client.widget.Operation;
-import org.rstudio.core.client.widget.OperationWithInput;
+import org.rstudio.core.client.widget.ProgressIndicator;
+import org.rstudio.core.client.widget.ProgressOperationWithInput;
+import org.rstudio.studio.client.application.ApplicationInterrupt;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.common.DelayedProgressRequestCallback;
 import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.common.GlobalProgressDelayer;
-import org.rstudio.studio.client.common.SimpleRequestCallback;
-import org.rstudio.studio.client.common.console.ConsoleProcess;
-import org.rstudio.studio.client.common.console.ProcessExitEvent;
 import org.rstudio.studio.client.server.VoidServerRequestCallback;
 import org.rstudio.studio.client.workbench.WorkbenchListManager;
 import org.rstudio.studio.client.workbench.WorkbenchView;
@@ -55,6 +53,7 @@ import org.rstudio.studio.client.workbench.views.connections.events.ActiveConnec
 import org.rstudio.studio.client.workbench.views.connections.events.ConnectionListChangedEvent;
 import org.rstudio.studio.client.workbench.views.connections.events.ConnectionOpenedEvent;
 import org.rstudio.studio.client.workbench.views.connections.events.ConnectionUpdatedEvent;
+import org.rstudio.studio.client.workbench.views.connections.events.ExecuteConnectionActionEvent;
 import org.rstudio.studio.client.workbench.views.connections.events.ExploreConnectionEvent;
 import org.rstudio.studio.client.workbench.views.connections.events.PerformConnectionEvent;
 import org.rstudio.studio.client.workbench.views.connections.events.ViewConnectionDatasetEvent;
@@ -62,15 +61,11 @@ import org.rstudio.studio.client.workbench.views.connections.model.Connection;
 import org.rstudio.studio.client.workbench.views.connections.model.ConnectionId;
 import org.rstudio.studio.client.workbench.views.connections.model.ConnectionOptions;
 import org.rstudio.studio.client.workbench.views.connections.model.ConnectionsServerOperations;
-import org.rstudio.studio.client.workbench.views.connections.model.NewSparkConnectionContext;
-import org.rstudio.studio.client.workbench.views.connections.model.SparkVersion;
-import org.rstudio.studio.client.workbench.views.connections.ui.InstallInfoPanel;
-import org.rstudio.studio.client.workbench.views.connections.ui.ComponentsNotInstalledDialogs;
-import org.rstudio.studio.client.workbench.views.connections.ui.NewSparkConnectionDialog;
+import org.rstudio.studio.client.workbench.views.connections.model.NewConnectionContext;
+import org.rstudio.studio.client.workbench.views.connections.ui.NewConnectionWizard;
 import org.rstudio.studio.client.workbench.views.console.events.SendToConsoleEvent;
 import org.rstudio.studio.client.workbench.views.source.events.NewDocumentWithCodeEvent;
 import org.rstudio.studio.client.workbench.views.source.model.SourcePosition;
-import org.rstudio.studio.client.workbench.views.vcs.common.ConsoleProgressDialog;
 
 public class ConnectionsPresenter extends BasePresenter  
                                   implements PerformConnectionEvent.Handler,
@@ -91,6 +86,9 @@ public class ConnectionsPresenter extends BasePresenter
       HandlerRegistration addExploreConnectionHandler(
                                        ExploreConnectionEvent.Handler handler); 
       
+      HandlerRegistration addExecuteConnectionActionHandler(
+            ExecuteConnectionActionEvent.Handler handler);
+      
       void showConnectionExplorer(Connection connection, String connectVia);
       void setExploredConnection(Connection connection);
       
@@ -101,7 +99,7 @@ public class ConnectionsPresenter extends BasePresenter
       String getConnectVia();
       String getConnectCode();
       
-      void showConnectionProgress();
+      void showConnectionProgress(String message);
    }
    
    public interface Binder extends CommandBinder<Commands, ConnectionsPresenter> {}
@@ -115,7 +113,8 @@ public class ConnectionsPresenter extends BasePresenter
                                Binder binder,
                                final Commands commands,
                                WorkbenchListManager listManager,
-                               Session session)
+                               Session session,
+                               ApplicationInterrupt applicationInterrupt)
    {
       super(display);
       binder.bind(commands, this);
@@ -125,6 +124,7 @@ public class ConnectionsPresenter extends BasePresenter
       uiPrefs_ = uiPrefs;
       globalDisplay_ = globalDisplay;
       eventBus_ = eventBus;
+      applicationInterrupt_ = applicationInterrupt;
          
       // search filter
       display_.addSearchFilterChangeHandler(new ValueChangeHandler<String>() {
@@ -152,7 +152,20 @@ public class ConnectionsPresenter extends BasePresenter
          {
             showAllConnections(true);
          }
+      });
+      
+      
+      display_.addExecuteConnectionActionHandler(
+            new ExecuteConnectionActionEvent.Handler()
+      {
          
+         @Override
+         public void onExecuteConnectionAction(
+               ExecuteConnectionActionEvent event)
+         {
+            server_.connectionExecuteAction(event.getConnectionId(), 
+                  event.getAction(), new VoidServerRequestCallback());
+         }
       });
       
       // events
@@ -248,123 +261,53 @@ public class ConnectionsPresenter extends BasePresenter
    {
       updateActiveConnections(event.getActiveConnections());
    }
+
+   private void showError(String errorMessage)
+   {
+      globalDisplay_.showErrorMessage("Error", errorMessage);
+   }
    
    public void onNewConnection()
    {
+      // if r session bussy, fail
+      if (commands_.interruptR().isEnabled()) {
+        showError(
+          "The R session is currently busy. Wait for completion or " +
+          "interrupt the current session and retry.");
+        return;
+      }
+
       // get the context
-      server_.getNewSparkConnectionContext(
-         new DelayedProgressRequestCallback<NewSparkConnectionContext>(
+      server_.getNewConnectionContext(
+         new DelayedProgressRequestCallback<NewConnectionContext>(
                                                    "New Connection...") {
    
             @Override
-            protected void onSuccess(final NewSparkConnectionContext context)
+            protected void onSuccess(final NewConnectionContext context)
             {
-               // prompt for no java installed
-               if (!context.isJavaInstalled())
-               {
-                  ComponentsNotInstalledDialogs.showJavaNotInstalled(context.getJavaInstallUrl());
-               }
+                // show dialog
+               NewConnectionWizard newConnectionWizard = new NewConnectionWizard(
+                 context,
+                 new ProgressOperationWithInput<ConnectionOptions>() {
+                    @Override
+                    public void execute(ConnectionOptions result,
+                                        ProgressIndicator indicator)
+                    {
+                       indicator.onCompleted();
+
+                       eventBus_.fireEvent(new PerformConnectionEvent(
+                          result.getConnectVia(),
+                          result.getConnectCode())
+                       );
+                    }
+                 }
+               );
                
-               // prompt for no spark installed
-               else if (!context.getLocalConnectionsSupported() &&
-                        !context.getClusterConnectionsSupported())
-               {
-                  ComponentsNotInstalledDialogs.showSparkHomeNotDefined();
-               }
-               
-               // otherwise proceed with connecting
-               else
-               {
-                  // show dialog
-                  new NewSparkConnectionDialog(
-                   context,
-                   new OperationWithInput<ConnectionOptions>() {
-                     @Override
-                     public void execute(final ConnectionOptions result)
-                     {
-                        withRequiredSparkInstallation(
-                              result.getSparkVersion(),
-                              result.getRemote(),
-                              context.getSparkHome() != null,
-                              new Command() {
-                                 @Override
-                                 public void execute()
-                                 {
-                                    eventBus_.fireEvent(new PerformConnectionEvent(
-                                          result.getConnectVia(),
-                                          result.getConnectCode()));
-                                 }
-                                 
-                              });
-                     }
-                  }).showModal();
-               }
+               newConnectionWizard.showModal();
             }
          });      
    }
    
-   
-   private void withRequiredSparkInstallation(final SparkVersion sparkVersion,
-                                              boolean remote,
-                                              boolean hasSparkHome,
-                                              final Command command)
-   {
-      // if there is no spark version then just execute immediately
-      if (sparkVersion == null)
-      {
-         command.execute();
-         return;
-      }
-      
-      if (!sparkVersion.isInstalled() && !hasSparkHome)
-      {
-         globalDisplay_.showYesNoMessage(
-            MessageDialog.QUESTION, 
-            "Install Spark Components",
-            InstallInfoPanel.getInfoText(sparkVersion, remote, true),
-            false,
-            new Operation() {  public void execute() {
-               server_.installSpark(
-                 sparkVersion.getSparkVersionNumber(),
-                 sparkVersion.getHadoopVersionNumber(),
-                 new SimpleRequestCallback<ConsoleProcess>(){
-
-                    @Override
-                    public void onResponseReceived(ConsoleProcess process)
-                    {
-                       final ConsoleProgressDialog dialog = 
-                             new ConsoleProgressDialog(process, server_);
-                       dialog.showModal();
-           
-                       process.addProcessExitHandler(
-                          new ProcessExitEvent.Handler()
-                          {
-                             @Override
-                             public void onProcessExit(ProcessExitEvent event)
-                             {
-                                if (event.getExitCode() == 0)
-                                {
-                                   dialog.hide();
-                                   command.execute();
-                                } 
-                             }
-                          }); 
-                    }
-
-               });   
-            }},
-            null,
-            null,
-            "Install",
-            "Cancel",
-            true);
-      }
-      else
-      {
-         command.execute();
-      }
-   }
-   
    @Override
    public void onPerformConnection(PerformConnectionEvent event)
    {
@@ -381,7 +324,7 @@ public class ConnectionsPresenter extends BasePresenter
          eventBus_.fireEvent(
                new SendToConsoleEvent(connectCode, true));
          
-         display_.showConnectionProgress();
+         display_.showConnectionProgress("Connecting");
       }
       else if (connectVia.equals(ConnectionOptions.CONNECT_NEW_R_SCRIPT) ||
                connectVia.equals(ConnectionOptions.CONNECT_NEW_R_NOTEBOOK))
@@ -416,7 +359,7 @@ public class ConnectionsPresenter extends BasePresenter
          eventBus_.fireEvent(
             new NewDocumentWithCodeEvent(type, code, cursorPosition, true));
          
-         display_.showConnectionProgress();
+         display_.showConnectionProgress("Connecting");
       }
    }
    
@@ -429,9 +372,9 @@ public class ConnectionsPresenter extends BasePresenter
       GlobalProgressDelayer progress = new GlobalProgressDelayer(
                               globalDisplay_, 100, "Previewing table...");
       
-      server_.connectionPreviewTable(
-         exploredConnection_, 
-         event.getDataset(),
+      server_.connectionPreviewObject(
+         exploredConnection_.getId(), 
+         event.getDataset().createSpecifier(),
          new VoidServerRequestCallback(progress.getIndicator())); 
    }
    
@@ -441,23 +384,38 @@ public class ConnectionsPresenter extends BasePresenter
       if (exploredConnection_ == null)
          return;
       
+      // protect the connection from interleaving actions while the dialog is up
+      final Connection removingConnection = exploredConnection_;
+      exploredConnection_ = null;
+      
       globalDisplay_.showYesNoMessage(
          MessageDialog.QUESTION,
          "Remove Connection",
          "Are you sure you want to remove this connection from the connection history?",
          new Operation() {
-
             @Override
             public void execute()
             {
                server_.removeConnection(
-                 exploredConnection_.getId(), new VoidServerRequestCallback()); 
-               disconnectConnection(false);
-               showAllConnections(true);
+                 removingConnection.getId(), 
+                 new VoidServerRequestCallback()
+                 {
+                    @Override
+                    protected void onSuccess()
+                    {
+                        exploredConnection_ = removingConnection;
+                        disconnectConnection(false);
+                        showAllConnections(true);
+                    }
+                    @Override
+                    protected void onFailure()
+                    {
+                        exploredConnection_ = removingConnection;
+                    }
+                 }); 
             }
          },
          true);
-     
    }
    
    @Handler
@@ -476,21 +434,15 @@ public class ConnectionsPresenter extends BasePresenter
          @Override
          public void execute()
          {
-            server_.getDisconnectCode(exploredConnection_, 
-                  new SimpleRequestCallback<String>() {
-               @Override
-               public void onResponseReceived(String disconnectCode)
-               {
-                  eventBus_.fireEvent(new SendToConsoleEvent(disconnectCode, true));
-               }
-          });  
+            server_.connectionDisconnect(exploredConnection_.getId(), 
+                  new VoidServerRequestCallback());
          }  
       };
       
       if (prompt)
       {
          StringBuilder builder = new StringBuilder();
-         builder.append("Are you sure you want to disconnect from Spark?");
+         builder.append("Are you sure you want to disconnect?");
          globalDisplay_.showYesNoMessage(
                MessageDialog.QUESTION,
                "Disconnect",
@@ -514,36 +466,6 @@ public class ConnectionsPresenter extends BasePresenter
       display_.updateExploredConnection("");
    }
    
-   
-   @Handler
-   public void onSparkLog()
-   {
-      if (exploredConnection_ == null)
-         return;
-      
-      server_.showSparkLog(exploredConnection_, 
-                           new VoidServerRequestCallback());
-      
-   }
-   
-   @Handler
-   public void onSparkUI()
-   {
-      if (exploredConnection_ == null)
-         return;
-      
-      server_.showSparkUI(exploredConnection_, 
-                          new VoidServerRequestCallback());
-      
-   }
-   
-   @Handler
-   public void onSparkHelp()
-   {
-      globalDisplay_.openRStudioLink("using_spark", false);
-   }
-   
-   
    private void showAllConnections(boolean animate)
    {
       exploredConnection_ = null;
@@ -598,16 +520,14 @@ public class ConnectionsPresenter extends BasePresenter
          boolean connected = isConnected(exploredConnection_.getId());
          commands_.removeConnection().setVisible(!connected);
          commands_.disconnectConnection().setVisible(connected);
-         commands_.sparkLog().setVisible(connected);
-         commands_.sparkUI().setVisible(connected);
+         // TODO: show connection actions
          commands_.refreshConnection().setVisible(connected);
       }
       else
       {
          commands_.removeConnection().setVisible(false);
          commands_.disconnectConnection().setVisible(false);
-         commands_.sparkLog().setVisible(false);
-         commands_.sparkUI().setVisible(false);
+         // TODO: hide connection actions
          commands_.refreshConnection().setVisible(false);
       }
    }
@@ -649,10 +569,11 @@ public class ConnectionsPresenter extends BasePresenter
    private final Commands commands_;
    private UIPrefs uiPrefs_;
    private final ConnectionsServerOperations server_ ;
+   @SuppressWarnings("unused") private final ApplicationInterrupt applicationInterrupt_;
    
    // client state
    public static final String MODULE_CONNECTIONS = "connections-pane";
-   private static final String KEY_EXPLORED_CONNECTION = "exploredConnection";
+   private static final String KEY_EXPLORED_CONNECTION = "exploredConnections";
    private Connection exploredConnection_;
    private Connection lastExploredConnection_;
    
