@@ -1,7 +1,7 @@
 /*
  * TextEditingTargetWidget.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2009-16 by RStudio, Inc.
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -46,6 +46,7 @@ import org.rstudio.core.client.events.EnsureVisibleEvent;
 import org.rstudio.core.client.events.EnsureVisibleHandler;
 import org.rstudio.core.client.events.MouseDragHandler;
 import org.rstudio.core.client.layout.RequiresVisibilityChanged;
+import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.theme.res.ThemeResources;
 import org.rstudio.core.client.widget.*;
 import org.rstudio.studio.client.RStudioGinjector;
@@ -55,6 +56,7 @@ import org.rstudio.studio.client.common.filetypes.FileTypeRegistry;
 import org.rstudio.studio.client.common.filetypes.TextFileType;
 import org.rstudio.studio.client.common.icons.StandardIcons;
 import org.rstudio.studio.client.rmarkdown.RmdOutput;
+import org.rstudio.studio.client.rmarkdown.events.RenderRmdEvent;
 import org.rstudio.studio.client.rmarkdown.events.RmdOutputFormatChangedEvent;
 import org.rstudio.studio.client.rsconnect.RSConnect;
 import org.rstudio.studio.client.rsconnect.ui.RSConnectPublishButton;
@@ -148,9 +150,10 @@ public class TextEditingTargetWidget
                double initialWidth_ = 0;
                
                @Override
-               public void beginDrag(MouseDownEvent event)
+               public boolean beginDrag(MouseDownEvent event)
                {
                   initialWidth_ = editorPanel_.getWidgetSize(docOutlineWidget_);
+                  return true;
                }
                
                @Override
@@ -158,7 +161,7 @@ public class TextEditingTargetWidget
                {
                   double initialWidth = initialWidth_;
                   double xDiff = event.getTotalDelta().getMouseX();
-                  double newSize = initialWidth + xDiff;
+                  double newSize = initialWidth - xDiff;
                   
                   // We allow an extra pixel here just to 'hide' the border
                   // if the outline is maximized, since the 'separator'
@@ -268,6 +271,7 @@ public class TextEditingTargetWidget
       toolbar.addLeftWidget(knitDocumentButton_);
       toolbar.addLeftWidget(compilePdfButton_ = commands_.compilePDF().createToolbarButton());
       rmdFormatButton_ = new ToolbarPopupMenuButton(false, true);
+      rmdFormatButton_.getMenu().setAutoOpen(true);
       toolbar.addLeftWidget(rmdFormatButton_);
       
       runDocumentMenuButton_ = new ToolbarPopupMenuButton(false, true);
@@ -275,13 +279,14 @@ public class TextEditingTargetWidget
       runDocumentMenuButton_.addSeparator();
       runDocumentMenuButton_.addMenuItem(commands_.clearPrerenderedOutput().createMenuItem(false), "");     
       toolbar.addLeftWidget(runDocumentMenuButton_);
+      runDocumentMenuButton_.setVisible(false);
       
       ToolbarPopupMenu rmdOptionsMenu = new ToolbarPopupMenu();
       rmdOptionsMenu.addItem(commands_.editRmdFormatOptions().createMenuItem(false));
       
       rmdOptionsButton_ = new ToolbarButton(
             null,  
-            StandardIcons.INSTANCE.options(),
+            new ImageResource2x(StandardIcons.INSTANCE.options2x()),
             rmdOptionsMenu, 
             false);
       
@@ -403,13 +408,14 @@ public class TextEditingTargetWidget
       {
          toolbar.addRightSeparator();
          publishButton_ = new RSConnectPublishButton(
+               RSConnectPublishButton.HOST_EDITOR,
                RSConnect.CONTENT_TYPE_APP, false, null);
          toolbar.addRightWidget(publishButton_);
       }
       
       toggleDocOutlineButton_ = new LatchingToolbarButton(
          "",
-            StandardIcons.INSTANCE.outline(),
+            new ImageResource2x(StandardIcons.INSTANCE.outline2x()),
             new ClickHandler()
             {
                @Override
@@ -449,12 +455,15 @@ public class TextEditingTargetWidget
                      @Override
                      protected void onComplete()
                      {
+                        if (destination == 0) editorPanel_.setWidgetSize(docOutlineWidget_, 0);
                         target_.setPreferredOutlineWidgetVisibility(destination != 0);
                      }
                   }.run(500);
                }
             });
       
+      toggleDocOutlineButton_.addStyleName("rstudio-themes-inverts");
+
       // Time-out setting the latch just to ensure the document outline
       // has actually been appropriately rendered.
       new Timer()
@@ -494,7 +503,7 @@ public class TextEditingTargetWidget
    {
       if (codeTransform_ == null)
       {
-         ImageResource icon = ThemeResources.INSTANCE.codeTransform();
+         ImageResource icon = new ImageResource2x(ThemeResources.INSTANCE.codeTransform2x());
 
          ToolbarPopupMenu menu = new ToolbarPopupMenu();
          menu.addItem(commands_.codeCompletion().createMenuItem(false));
@@ -547,11 +556,18 @@ public class TextEditingTargetWidget
       boolean isScript = fileType.isScript();
       boolean isRMarkdown2 = extendedType_.equals("rmarkdown");
       boolean canPreviewFromR = fileType.canPreviewFromR();
+      boolean terminalAllowed = session_.getSessionInfo().getAllowShell();
+      
+      if (isScript && !terminalAllowed)
+      {
+         commands_.executeCode().setEnabled(false);
+         commands_.executeCodeWithoutFocus().setEnabled(false);
+      }
       
       // don't show the run buttons for cpp files, or R files in Shiny
       runButton_.setVisible(canExecuteCode && !canExecuteChunks && !isCpp && 
-            !isShinyFile());
-      runLastButton_.setVisible(runButton_.isVisible() && !canExecuteChunks);
+            !isShinyFile() && !(isScript && !terminalAllowed));
+      runLastButton_.setVisible(runButton_.isVisible() && !canExecuteChunks && !isScript);
       
       // show insertion options for various knitr engines in rmarkdown v2
       insertChunkMenu_.setVisible(isRMarkdown2);
@@ -569,7 +585,7 @@ public class TextEditingTargetWidget
       else
          srcOnSaveLabel_.setText("Source on Save");
       codeTransform_.setVisible(
-            (canExecuteCode && !fileType.canAuthorContent()) ||
+            (canExecuteCode && !isScript && !fileType.canAuthorContent()) ||
             fileType.isC() || fileType.isStan());   
      
       sourceButton_.setVisible(canSource && !isPlainMarkdown);
@@ -883,7 +899,35 @@ public class TextEditingTargetWidget
                                       true,
                                       cmd); 
          rmdFormatButton_.addMenuItem(item, knitWithParams.getMenuLabel(false));
+      }
       
+      if (session_.getSessionInfo().getKnitWorkingDirAvailable())
+      {
+         MenuBar knitDirMenu = new MenuBar(true);
+         DocPropMenuItem knitInDocDir = new DocShadowPropMenuItem(
+               "Document Directory", 
+               docUpdateSentinel_, 
+               uiPrefs_.knitWorkingDir(), 
+               RenderRmdEvent.WORKING_DIR_PROP,
+               UIPrefsAccessor.KNIT_DIR_DEFAULT);
+         knitDirMenu.addItem(knitInDocDir);
+         DocPropMenuItem knitInProjectDir = new DocShadowPropMenuItem(
+               "Project Directory", 
+               docUpdateSentinel_, 
+               uiPrefs_.knitWorkingDir(), 
+               RenderRmdEvent.WORKING_DIR_PROP,
+               UIPrefsAccessor.KNIT_DIR_PROJECT);
+         knitDirMenu.addItem(knitInProjectDir);
+         DocPropMenuItem knitInCurrentDir = new DocShadowPropMenuItem(
+               "Current Working Directory", 
+               docUpdateSentinel_, 
+               uiPrefs_.knitWorkingDir(), 
+               RenderRmdEvent.WORKING_DIR_PROP,
+               UIPrefsAccessor.KNIT_DIR_CURRENT);
+         knitDirMenu.addItem(knitInCurrentDir);
+
+         rmdFormatButton_.addSeparator();
+         rmdFormatButton_.addMenuItem(knitDirMenu, "Knit Directory");
       }
       
       addClearKnitrCacheMenu(rmdFormatButton_);
@@ -893,7 +937,6 @@ public class TextEditingTargetWidget
      
       if (publishButton_ != null)
          publishButton_.setIsStatic(true);
-      isShiny_ = false;
    }
    
    private void addClearKnitrCacheMenu(ToolbarPopupMenuButton menuButton)
@@ -932,12 +975,11 @@ public class TextEditingTargetWidget
             DomUtils.htmlToText(
                   commands_.knitDocument().getShortcutPrettyHtml()) + ")");
       knitDocumentButton_.setText(knitCommandText_);
-      knitDocumentButton_.setLeftImage(StandardIcons.INSTANCE.run());
+      knitDocumentButton_.setLeftImage(new ImageResource2x(StandardIcons.INSTANCE.run2x()));
       
       runDocumentMenuButton_.setVisible(isShinyPrerendered);
       setKnitDocumentMenuVisible(isShinyPrerendered);
       
-      isShiny_ = true;
       if (publishButton_ != null)
          publishButton_.setIsStatic(false);
    }
@@ -992,7 +1034,10 @@ public class TextEditingTargetWidget
          }
          else if (type == SourceDocument.XT_RMARKDOWN)
          {
-            publishButton_.setRmd(publishPath, !isShiny_);
+            // don't publish markdown docs as static
+            publishButton_.setRmd(publishPath, 
+                  false // not static
+                  );
          }
          else 
          {
@@ -1165,17 +1210,17 @@ public class TextEditingTargetWidget
             DocUpdateSentinel.PROPERTY_TRUE));
       menu.addSeparator();
       
-      if (uiPrefs_.showRmdChunkOutputInline().getValue() &&
-          type != RmdOutput.TYPE_SHINY)
+      if (type != RmdOutput.TYPE_SHINY)
       {
+        boolean inline = uiPrefs_.showRmdChunkOutputInline().getValue();
         menu.addItem(new DocPropMenuItem(
               "Chunk Output Inline", docUpdateSentinel_,
-              true,
+              inline,
               TextEditingTargetNotebook.CHUNK_OUTPUT_TYPE,
               TextEditingTargetNotebook.CHUNK_OUTPUT_INLINE));
         menu.addItem(new DocPropMenuItem(
               "Chunk Output in Console", docUpdateSentinel_,
-              false,
+              !inline,
               TextEditingTargetNotebook.CHUNK_OUTPUT_TYPE,
               TextEditingTargetNotebook.CHUNK_OUTPUT_CONSOLE));
          
@@ -1247,5 +1292,4 @@ public class TextEditingTargetWidget
    private String sourceCommandText_ = "Source";
    private String knitCommandText_ = "Knit";
    private String previewCommandText_ = "Preview";
-   private boolean isShiny_ = false;
 }
