@@ -402,6 +402,7 @@ public class DocUpdateSentinel
       {
          if (path != null)
          {
+            // notify that a save is underway (for collaborative editing)
             eventBus_.fireEvent(new SaveInitiatedEvent(path, getId()));
          }
       }
@@ -578,6 +579,13 @@ public class DocUpdateSentinel
       return properties.getString(propertyName);
    }
    
+   public String getProperty(String propertyName, String defaultValue)
+   {
+      if (hasProperty(propertyName))
+         return getProperty(propertyName);
+      return defaultValue;
+   }
+   
    public boolean getBoolProperty(String propertyName, boolean defaultValue)
    {
       if (hasProperty(propertyName))
@@ -753,7 +761,7 @@ public class DocUpdateSentinel
    {
       return sourceDoc_.getId();
    }
-
+   
    private boolean changesPending_ = false;
    private final ChangeTracker changeTracker_;
    private final SourceServerOperations server_;
