@@ -14,6 +14,7 @@
  */
 package org.rstudio.studio.client.workbench.views.vcs.git.dialog;
 
+import com.google.gwt.core.client.JsArray;
 import com.google.gwt.core.client.Scheduler;
 import com.google.gwt.core.client.Scheduler.ScheduledCommand;
 import com.google.gwt.dom.client.NativeEvent;
@@ -51,6 +52,7 @@ import org.rstudio.studio.client.common.vcs.DiffResult;
 import org.rstudio.studio.client.common.vcs.GitServerOperations;
 import org.rstudio.studio.client.common.vcs.GitServerOperations.PatchMode;
 import org.rstudio.studio.client.common.vcs.StatusAndPath;
+import org.rstudio.studio.client.common.vcs.StatusAndPathInfo;
 import org.rstudio.studio.client.server.ServerError;
 import org.rstudio.studio.client.server.ServerRequestCallback;
 import org.rstudio.studio.client.server.Void;
@@ -58,6 +60,7 @@ import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.model.ClientState;
 import org.rstudio.studio.client.workbench.model.Session;
 import org.rstudio.studio.client.workbench.model.helper.IntStateValue;
+import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
 import org.rstudio.studio.client.workbench.views.files.events.FileChangeEvent;
 import org.rstudio.studio.client.workbench.views.files.events.FileChangeHandler;
 import org.rstudio.studio.client.workbench.views.vcs.common.ChangelistTable;
@@ -74,6 +77,8 @@ import org.rstudio.studio.client.workbench.views.vcs.git.GitPresenterCore;
 import org.rstudio.studio.client.workbench.views.vcs.git.model.GitState;
 
 import java.util.ArrayList;
+import java.util.HashSet;
+import java.util.Set;
 
 public class GitReviewPresenter implements ReviewPresenter
 {
@@ -87,6 +92,7 @@ public class GitReviewPresenter implements ReviewPresenter
 
       HasValue<Boolean> getStagedCheckBox();
       HasValue<Boolean> getUnstagedCheckBox();
+      HasValue<Boolean> getIgnoreWhitespaceCheckBox();
       LineTablePresenter.Display getLineTableDisplay();
       ChangelistTable getChangelistTable();
       HasValue<Integer> getContextLines();
@@ -215,6 +221,7 @@ public class GitReviewPresenter implements ReviewPresenter
                              final GitState gitState,
                              final Session session,
                              final GlobalDisplay globalDisplay,
+                             final UIPrefs uiPrefs,
                              VCSFileOpener vcsFileOpener)
    {
       gitPresenterCore_ = gitPresenterCore;
@@ -222,10 +229,12 @@ public class GitReviewPresenter implements ReviewPresenter
       view_ = view;
       globalDisplay_ = globalDisplay;
       gitState_ = gitState;
+      uiPrefs_ = uiPrefs;
       vcsFileOpener_ = vcsFileOpener;
+      gitCommitLargeFileSize_ = session.getSessionInfo().getGitCommitLargeFileSize();
       
       binder.bind(commands, this);
-
+      
       new WidgetHandlerRegistration(view.asWidget())
       {
          @Override
@@ -282,7 +291,7 @@ public class GitReviewPresenter implements ReviewPresenter
             });
          }
       };
-
+      
       view_.getChangelistTable().addSelectionChangeHandler(new SelectionChangeEvent.Handler()
       {
          @Override
@@ -476,6 +485,7 @@ public class GitReviewPresenter implements ReviewPresenter
       });
       view_.getUnstageAllButton().addClickHandler(
             new ApplyPatchClickHandler(PatchMode.Stage, true));
+      
       view_.getStagedCheckBox().addValueChangeHandler(
             new ValueChangeHandler<Boolean>()
             {
@@ -486,6 +496,23 @@ public class GitReviewPresenter implements ReviewPresenter
                      updateDiff(false);
                }
             });
+      
+      view_.getIgnoreWhitespaceCheckBox().setValue(
+            uiPrefs_.gitDiffIgnoreWhitespace().getGlobalValue());
+      
+      view_.getIgnoreWhitespaceCheckBox().addValueChangeHandler(
+            new ValueChangeHandler<Boolean>()
+            {
+               @Override
+               public void onValueChange(ValueChangeEvent<Boolean> event)
+               {
+                  boolean value = event.getValue();
+                  uiPrefs_.gitDiffIgnoreWhitespace().setGlobalValue(value);
+                  uiPrefs_.writeUIPrefs();
+                  updateDiff(false);
+               }
+            });
+      
       view_.getLineTableDisplay().addDiffChunkActionHandler(new ApplyPatchHandler());
       view_.getLineTableDisplay().addDiffLineActionHandler(new ApplyPatchHandler());
 
@@ -519,6 +546,100 @@ public class GitReviewPresenter implements ReviewPresenter
       {
          @Override
          public void onClick(ClickEvent event)
+         {
+            // record the current selected paths (as a set)
+            final Set<String> selectedPaths = new HashSet<String>();
+            selectedPaths.addAll(view_.getSelectedPaths());
+            
+            // first, double-check the file sizes of any files that are going
+            // to be added or modified (help the user avoid committing large
+            // files to the repository)
+            server_.gitFullStatus(new ServerRequestCallback<JsArray<StatusAndPathInfo>>()
+            {
+               @Override
+               public void onResponseReceived(JsArray<StatusAndPathInfo> response)
+               {
+                  // if this failed for some reason just go ahead with the commit
+                  if (response == null)
+                  {
+                     onCommit();
+                     return;
+                  }
+                  
+                  // iterate through the files and check for large modifications
+                  for (int i = 0, n = response.length(); i < n; i++)
+                  {
+                     StatusAndPathInfo info = response.get(i);
+                     
+                     // skip if this isn't a currently selected file for add
+                     if (!selectedPaths.contains(info.getPath()))
+                        continue;
+                     
+                     String status = info.getStatus();
+                     if (status.charAt(0) != 'A')
+                        continue;
+                     
+                     // warn if we're trying to commit a file >10MB in size
+                     double size = info.getFileSize();
+                     if (gitCommitLargeFileSize_ > 0 && size >= gitCommitLargeFileSize_)
+                     {
+                        onLargeFile();
+                        return;
+                     }
+                  }
+                  
+                  // no large files; proceed with commit as normal
+                  onCommit();
+                  return;
+               }
+               
+               @Override
+               public void onError(ServerError error)
+               {
+                  Debug.logError(error);
+                  
+                  // although server errors here are unexpected,
+                  // make a best effort attempt to commit regardless
+                  onCommit();
+               }
+            });
+         }
+         
+         private void onLargeFile()
+         {
+            String prettySize = StringUtil.formatFileSize(gitCommitLargeFileSize_);
+            String message =
+                  "Some of the files to be committed are quite large " +
+                  "(>" + prettySize + " in size). Are you sure you want to commit these files?";
+            
+            globalDisplay_.showYesNoMessage(
+                  GlobalDisplay.MSG_WARNING,
+                  "Committing Large Files",
+                  message,
+                  false,
+                  new Operation()
+                  {
+                     @Override
+                     public void execute()
+                     {
+                        onCommit();
+                     }
+                  },
+                  new Operation()
+                  {
+                     @Override
+                     public void execute()
+                     {
+                        // no-op on cancel
+                     }
+                  },
+                  null,
+                  "Commit",
+                  "Cancel",
+                  true);
+         }
+         
+         private void onCommit()
          {
             server_.gitCommit(
                   view_.getCommitMessage().getText(),
@@ -562,6 +683,8 @@ public class GitReviewPresenter implements ReviewPresenter
                      }
                   });
          }
+         
+         
       });
 
       view_.getOverrideSizeWarningButton().addClickHandler(new ClickHandler()
@@ -672,6 +795,7 @@ public class GitReviewPresenter implements ReviewPresenter
             patchMode,
             view_.getContextLines().getValue(),
             overrideSizeWarning_,
+            uiPrefs_.gitDiffIgnoreWhitespace().getValue(),
             new SimpleRequestCallback<DiffResult>("Diff Error")
             {
                @Override
@@ -807,11 +931,13 @@ public class GitReviewPresenter implements ReviewPresenter
    // Hack to prevent us flipping to unstaged view when a line is unstaged
    // from staged view
    private boolean softModeSwitch_;
-   private GitState gitState_;
+   private final GitState gitState_;
+   private final UIPrefs uiPrefs_;
    private final VCSFileOpener vcsFileOpener_;
    private boolean initialized_;
    private static final String MODULE_GIT = "vcs_git";
    private static final String KEY_CONTEXT_LINES = "context_lines";
+   private final int gitCommitLargeFileSize_;
 
    private boolean overrideSizeWarning_ = false;
 }
